/*
 * Copyright 2013 Kerio Technologies s.r.o. All rights reserved.
 */
package com.kerio.im.bean;

/**
 * Object storing information about KMS/XMPP user. Equals and hashCode methods 
 * counts only with domain and login.
 * @author tskoda
 */
public class User {

    /**
     * Username (without domain part).
     */
    private final String login;
    /**
     * Users primary domain.
     */
    private final String domain;
    /**
     * Users full name.
     */
    private final String fullName;
    /**
     * User primary email in form login@domain.
     */
    private final String primaryEmail;
    /**
     * If user is enabled or disabled.
     */
    private final boolean enabled;
    /**
     * If user is public (enabled in GAL) or private.
     */
    private final boolean visible; 
    
    /**
     * If user can send messages outside his/her own domain.
     */
    private final boolean allowedSendOutside;
    
    public User(String login, String domain, String fullName, boolean enabled, boolean visible, boolean allowedSendOutside) {
        this.login = login;
        this.domain = domain;
        this.primaryEmail = login + "@" + domain;
        this.fullName = fullName;
        this.enabled = enabled;
        this.visible = visible;
        this.allowedSendOutside = allowedSendOutside;
    }

    public boolean isEnabled() {
        return enabled;
    }

    public String getDomain() {
        return domain;
    }

    public String getLogin() {
        return login;
    }

    public String getFullName() {
        return fullName;
    }

    public String getPrimaryEmail() {
        return primaryEmail;
    }

    public boolean isVisible() {
        return visible;
    }

    public boolean isAllowedSendOutside() {
        return allowedSendOutside;
    }

    @Override
    public String toString() {
        return "User [" + login + '@' + domain + ':' + fullName + ':' + enabled + ':' + visible + ']';
    }

    @Override
    public boolean equals(Object obj) {
        if (obj == null) {
            return false;
        }
        if (getClass() != obj.getClass()) {
            return false;
        }
        final User other = (User) obj;
        if ((this.domain == null) ? (other.domain != null) : !this.domain.equals(other.domain)) {
            return false;
        }
        if ((this.login == null) ? (other.login != null) : !this.login.equals(other.login)) {
            return false;
        }
        return true;
    }

    @Override
    public int hashCode() {
        int hash = 3;
        hash = 37 * hash + (this.domain != null ? this.domain.hashCode() : 0);
        hash = 37 * hash + (this.login != null ? this.login.hashCode() : 0);
        return hash;
    }
}
