/*
 * Copyright 2013 Kerio Technologies s.r.o. All rights reserved.
 */
package com.kerio.im.common;

import java.util.*;

/**
 *
 * @author tskoda
 */
public class CollectionUtil {
   
    public static <T> void sync(
            Collection<T> from,
            Collection<T> to,
            Function<T> onAdd,
            Function<T> onRemove,
            Function<T> onKeep) {

        Set<T> toDelete = new HashSet<T>(to);

        for (T fromElement : from) {

            if (to.contains(fromElement)) {
                toDelete.remove(fromElement);
                onKeep.ex(fromElement);
            } else {
                onAdd.ex(fromElement);

            }
        }

        if (onRemove != null) {
            for (T tElement : toDelete) {
                onRemove.ex(tElement);
            }
        }
    }

    public static <F, T> void map(Collection<F> from, Mapper<F, T> mapper, Collection<T> to) {
        for (F item : from) {
            //mapper can return null, omit this values by desing
            T mappedItem = mapper.map(item);
            if (mappedItem != null) {
                to.add(mapper.map(item));
            }
        }
    }
}
