/*
 * Copyright 2013 Kerio Technologies s.r.o. All rights reserved.
 */
package com.kerio.im.common;

import java.io.File;
import java.io.IOException;

/**
 *
 * @author tskoda
 */
public final class FileUtil {

    private FileUtil() {
    }
    

    public static File createParent(File outputFile) throws IOException {

        File parent = outputFile.getParentFile();

        if (!parent.exists() && !parent.mkdir()) {
            throw new IOException("Can not create path for " + outputFile);
        }

        return outputFile;
    }

    /**
     * Deletes folder/file if exists.
     *
     * @param folder parent folder of file
     * @param file file to delete
     * @return false only if file exists and cant be deleted, otherwise true
     * @throws IOException
     */
    public static boolean deleteFile(final File folder, String file) throws IOException {

        boolean result = true;

        if (folder != null) {
            File fileToDelete = new File(folder, file);
            if (fileToDelete.exists()) { //if not exist return true
                if (fileToDelete.isFile()) {
                    result = fileToDelete.delete();
                } else {
                    throw new IOException("Failed to delete file, file " + fileToDelete.getName() + " is a folder");
                }
            }
        }
        return result;
    }

    /**
     * Delete folder only if is empty.
     *
     * @param folder folder to delete
     * @return false only if folder exists and cant be deleted, otherwise true
     * @throws IOException when folder represents a file
     */
    public static boolean deleteFolder(final File folder) throws IOException {
        boolean result = true;

        if ((folder != null) && (folder.exists())) {
            if (folder.isDirectory()) {
                result = folder.delete();
            } else {
                throw new IOException("Failed to delete folder, folder " + folder.getName() + " is not a directory");
            }
        }

        return result;
    }

    /**
     * Delete all files contained in folder and folder itself if it doesnt
     * contain any subfolder.
     *
     * @param folder to delete
     * @return false only if folder was not deleted completely (some files were
     * locked or subdirectory exist).
     * @throws IOException only if folder does not represents directory
     */
    public static boolean deleteFolderWithFiles(final File folder) throws IOException {
        boolean result = true;

        if (folder != null && folder.exists()) {
            if (folder.isDirectory()) {
                for (File child : folder.listFiles()) {
                    if (child.isFile()) {
                        result = child.delete() && result;
                    } else {
                        result = false;
                    }
                }
                result = result && folder.delete();
            } else {
                throw new IOException("Failed to delete folder, folder " + folder.getName() + " is not a directory");
            }
        }

        return result;
    }
}
