/*
 * Copyright 2013 Kerio Technologies s.r.o. All rights reserved.
 */
package com.kerio.im.common;

import java.io.ByteArrayOutputStream;
import java.io.IOException;
import java.util.logging.Level;
import java.util.logging.Logger;

/**
 * Inspired by http://blogs.oracle.com/nickstephen/entry/java_redirecting_system_out_and.
 */
/**
 *
 * @author development
 */
public class OutputStreamToLogger extends ByteArrayOutputStream {
    private static final String lineSeparator = System.getProperty("line.separator");
    private final Logger log;
    private final Level level;

    public OutputStreamToLogger(Logger log, Level level) {
        this.log = log;
        this.level = level;
    }

    @Override
    public void flush() throws IOException {

        synchronized (this) {
            try {
                super.flush();
                String record = this.toString();
                super.reset();
                if (record.length() == 0 || record.equals(lineSeparator)) {
                    // avoid empty records
                    return;
                }
                log.logp(level, log.getName(), "<unknown>", record);
            } catch (Exception e) {
                //there is no place where to log errors when logging doesnt work
            }
        }
    }
    
}
