/*
 * Copyright 2013 Kerio Technologies s.r.o. All rights reserved.
 */
package com.kerio.im.common.error;

import java.text.MessageFormat;

/**
 *
 * @author tskoda
 */
public class ServiceException extends RuntimeException {
    
    private final Code code;
    private final Object[] parameters;

    public ServiceException(String message, Code code) {
        super(message);
        this.code = code;
        this.parameters = null;
    }
    
    public ServiceException(String message, Code code, Object... parameters) {
        super(message);
        this.code = code;
        this.parameters = parameters;
    }

    public ServiceException(String message, Throwable cause, Code code) {
        super(message, cause);
        this.code = code;
        this.parameters = null;
    }
    
    public ServiceException(String message, Throwable cause, Code code, Object... parameters) {
        super(message, cause);
        this.code = code;
        this.parameters = parameters;
    }
    
    @Override
    public String getMessage() {
        if (parameters == null) {
            return super.getMessage();
        } else {
            return MessageFormat.format(super.getMessage(), parameters);
        }
    }
    
    public int getIntCode() {
        return code.ordinal();
    }
    
    public Code getCode() {
        return code;
    }
    
    
    
}
