package com.kerio.im.common;

import static org.junit.Assert.assertEquals;
import static org.junit.Assert.assertNotNull;
import static org.junit.Assert.assertNull;
import static org.mockito.AdditionalMatchers.or;
import static org.mockito.Matchers.isA;
import static org.mockito.Matchers.isNull;

import com.diffblue.deeptestutils.Reflector;
import java.io.File;
import java.util.HashMap;
import org.junit.Rule;
import org.junit.Test;
import org.junit.rules.ExpectedException;
import org.junit.rules.Timeout;
import org.junit.runner.RunWith;
import org.powermock.api.mockito.PowerMockito;
import org.powermock.core.classloader.annotations.PrepareForTest;
import org.powermock.modules.junit4.PowerMockRunner;

@RunWith(PowerMockRunner.class)
public class TypedPropertiesDiffblueTest {

  @Rule public final ExpectedException thrown = ExpectedException.none();
  @Rule public final Timeout globalTimeout = new Timeout(10000);

  /* testedClasses: TypedProperties */
  // Test generated by Diffblue Deeptest.

  @Test
  public void asCopyInputNotNullOutputNotNull() {

    // Arrange
    final TypedProperties from = new TypedProperties();
    final HashMap<Key, Object> hashMap = new HashMap<Key, Object>();
    Reflector.setField(from, "properties", hashMap);

    // Act
    final TypedProperties retval = TypedProperties.asCopy(from);

    // result
    assertNotNull(retval);
    final HashMap<Key, Object> hashMap1 = new HashMap<Key, Object>();
    assertEquals(hashMap1, retval.properties);
  }

  // Test generated by Diffblue Deeptest.

  @Test
  public void boolInputNotNull1FalseOutputFalse() throws Exception {

    // Arrange
    final Key<Boolean> key = (Key<Boolean>) Reflector.getInstance("com.kerio.im.common.Key");
    Reflector.setField(key, "key", null);
    final HashMap<String, String> properties = new HashMap<String, String>();
    properties.put(null, "");
    final Boolean defaultValue = false;

    // Act
    final Boolean retval = TypedProperties.bool(key, properties, defaultValue);

    // result
    assertEquals(new Boolean(false), retval);
  }

  // Test generated by Diffblue Deeptest.

  @Test
  public void boolInputNotNull1FalseOutputTrue() throws Exception {

    // Arrange
    final Key<Boolean> key = (Key<Boolean>) Reflector.getInstance("com.kerio.im.common.Key");
    Reflector.setField(key, "key", null);
    final HashMap<String, String> properties = new HashMap<String, String>();
    properties.put(null, "1");
    final Boolean defaultValue = false;

    // Act
    final Boolean retval = TypedProperties.bool(key, properties, defaultValue);

    // result
    assertEquals(new Boolean(true), retval);
  }

  // Test generated by Diffblue Deeptest.

  @Test
  public void boolInputNotNull1OutputFalse() throws Exception {

    // Arrange
    final Key<Boolean> key = (Key<Boolean>) Reflector.getInstance("com.kerio.im.common.Key");
    Reflector.setField(key, "key", null);
    final HashMap<String, String> properties = new HashMap<String, String>();
    properties.put(null, "1S31C  1");

    // Act
    final Boolean retval = TypedProperties.bool(key, properties);

    // result
    assertEquals(new Boolean(false), retval);
  }

  // Test generated by Diffblue Deeptest.

  @Test
  public void boolInputNotNull1OutputIllegalArgumentException() throws Exception {

    // Arrange
    final Key<Boolean> key = (Key<Boolean>) Reflector.getInstance("com.kerio.im.common.Key");
    Reflector.setField(key, "key", "!  !");
    final HashMap<String, String> properties = new HashMap<String, String>();
    properties.put(null, null);

    // Act
    thrown.expect(IllegalArgumentException.class);
    TypedProperties.bool(key, properties);

    // Method is not expected to return due to exception thrown
  }

  // Test generated by Diffblue Deeptest.

  @Test
  public void fileInputNotNull0NotNullOutputNotNull() throws Exception {

    // Arrange
    final Key<File> key = (Key<File>) Reflector.getInstance("com.kerio.im.common.Key");
    Reflector.setField(key, "key", "");
    final HashMap<String, String> properties = new HashMap<String, String>();
    final File defaultFile = PowerMockito.mock(File.class);
    Reflector.setField(defaultFile, "prefixLength", 0);
    Reflector.setField(defaultFile, "path", "");

    // Act
    final File retval = TypedProperties.file(key, properties, defaultFile);

    // result
    assertNotNull(retval);
    assertEquals(0, Reflector.getInstanceField(retval, "prefixLength"));
    assertEquals("", Reflector.getInstanceField(retval, "path"));
  }

  // Test generated by Diffblue Deeptest.

  @Test
  public void fileInputNotNull0OutputIllegalArgumentException() throws Exception {

    // Arrange
    final Key<File> key = (Key<File>) Reflector.getInstance("com.kerio.im.common.Key");
    Reflector.setField(key, "key", "#\"\"!");
    final HashMap<String, String> properties = new HashMap<String, String>();

    // Act
    thrown.expect(IllegalArgumentException.class);
    TypedProperties.file(key, properties);

    // Method is not expected to return due to exception thrown
  }

  // Test generated by Diffblue Deeptest.
  @PrepareForTest({File.class, TypedProperties.class})
  @Test
  public void fileInputNotNull1NotNullOutputNotNull() throws Exception {

    // Arrange
    final Key<File> key = (Key<File>) Reflector.getInstance("com.kerio.im.common.Key");
    Reflector.setField(key, "key", "");
    final HashMap<String, String> properties = new HashMap<String, String>();
    properties.put("", "");
    final File defaultFile = PowerMockito.mock(File.class);
    Reflector.setField(defaultFile, "prefixLength", 0);
    Reflector.setField(defaultFile, "path", null);
    final File file = PowerMockito.mock(File.class);
    Reflector.setField(file, "prefixLength", 0);
    Reflector.setField(file, "path", "");
    PowerMockito.whenNew(File.class)
        .withParameterTypes(String.class)
        .withArguments(or(isA(String.class), isNull(String.class)))
        .thenReturn(file);

    // Act
    final File retval = TypedProperties.file(key, properties, defaultFile);

    // result
    assertNotNull(retval);
    assertEquals(0, Reflector.getInstanceField(retval, "prefixLength"));
    assertEquals("", Reflector.getInstanceField(retval, "path"));
  }

  // Test generated by Diffblue Deeptest.
  @PrepareForTest({File.class, TypedProperties.class})
  @Test
  public void fileInputNotNull1OutputNotNull() throws Exception {

    // Arrange
    final Key<File> key = (Key<File>) Reflector.getInstance("com.kerio.im.common.Key");
    Reflector.setField(key, "key", null);
    final HashMap<String, String> properties = new HashMap<String, String>();
    properties.put(null, "        ");
    final File file = PowerMockito.mock(File.class);
    Reflector.setField(file, "prefixLength", 0);
    Reflector.setField(file, "path", "        ");
    PowerMockito.whenNew(File.class)
        .withParameterTypes(String.class)
        .withArguments(or(isA(String.class), isNull(String.class)))
        .thenReturn(file);

    // Act
    final File retval = TypedProperties.file(key, properties);

    // result
    assertNotNull(retval);
    assertEquals(0, Reflector.getInstanceField(retval, "prefixLength"));
    assertEquals("        ", Reflector.getInstanceField(retval, "path"));
  }

  // Test generated by Diffblue Deeptest.

  @Test
  public void getInputNotNullZeroOutputZero() throws Exception {

    // Arrange
    final TypedProperties objectUnderTest = new TypedProperties();
    final HashMap<Key, Object> hashMap = new HashMap<Key, Object>();
    hashMap.put(null, null);
    Reflector.setField(objectUnderTest, "properties", hashMap);
    final Key key = (Key) Reflector.getInstance("com.kerio.im.common.Key");
    Reflector.setField(key, "key", null);
    final Object defaultValue = 0;

    // Act
    final Object retval = objectUnderTest.get(key, defaultValue);

    // result
    assertEquals(new Integer(0), retval);
  }

  // Test generated by Diffblue Deeptest.

  @Test
  public void getInputNullNullOutputNull() {

    // Arrange
    final TypedProperties objectUnderTest = new TypedProperties();
    final HashMap<Key, Object> hashMap = new HashMap<Key, Object>();
    hashMap.put(null, null);
    Reflector.setField(objectUnderTest, "properties", hashMap);
    final Key key = null;
    final Object defaultValue = null;

    // Act
    final Object retval = objectUnderTest.get(key, defaultValue);

    // result
    assertNull(retval);
  }

  // Test generated by Diffblue Deeptest.

  @Test
  public void integerInputNotNull0OutputIllegalArgumentException() throws Exception {

    // Arrange
    final Key<Integer> key = (Key<Integer>) Reflector.getInstance("com.kerio.im.common.Key");
    Reflector.setField(key, "key", "!  !");
    final HashMap<String, String> properties = new HashMap<String, String>();

    // Act
    thrown.expect(IllegalArgumentException.class);
    TypedProperties.integer(key, properties);

    // Method is not expected to return due to exception thrown
  }

  // Test generated by Diffblue Deeptest.

  @Test
  public void integerInputNotNull1OutputPositive() throws Exception {

    // Arrange
    final Key<Integer> key = (Key<Integer>) Reflector.getInstance("com.kerio.im.common.Key");
    Reflector.setField(key, "key", null);
    final HashMap<String, String> properties = new HashMap<String, String>();
    properties.put(null, "7");

    // Act
    final Integer retval = TypedProperties.integer(key, properties);

    // result
    assertEquals(new Integer(7), retval);
  }

  // Test generated by Diffblue Deeptest.

  @Test
  public void integerInputNotNull1ZeroOutputPositive() throws Exception {

    // Arrange
    final Key<Integer> key = (Key<Integer>) Reflector.getInstance("com.kerio.im.common.Key");
    Reflector.setField(key, "key", "");
    final HashMap<String, String> properties = new HashMap<String, String>();
    properties.put("", "+7");
    final Integer defaultValue = 0;

    // Act
    final Integer retval = TypedProperties.integer(key, properties, defaultValue);

    // result
    assertEquals(new Integer(7), retval);
  }

  // Test generated by Diffblue Deeptest.

  @Test
  public void loongInputNotNull0OutputIllegalArgumentException() throws Exception {

    // Arrange
    final Key<Long> key = (Key<Long>) Reflector.getInstance("com.kerio.im.common.Key");
    Reflector.setField(key, "key", "!!");
    final HashMap<String, String> properties = new HashMap<String, String>();

    // Act
    thrown.expect(IllegalArgumentException.class);
    TypedProperties.loong(key, properties);

    // Method is not expected to return due to exception thrown
  }

  // Test generated by Diffblue Deeptest.

  @Test
  public void loongInputNotNull0ZeroOutputZero() throws Exception {

    // Arrange
    final Key<Long> key = (Key<Long>) Reflector.getInstance("com.kerio.im.common.Key");
    Reflector.setField(key, "key", null);
    final HashMap<String, String> properties = new HashMap<String, String>();
    final Long defaultValue = 0L;

    // Act
    final Long retval = TypedProperties.loong(key, properties, defaultValue);

    // result
    assertEquals(new Long(0L), retval);
  }

  // Test generated by Diffblue Deeptest.

  @Test
  public void loongInputNotNull1OutputPositive() throws Exception {

    // Arrange
    final Key<Long> key = (Key<Long>) Reflector.getInstance("com.kerio.im.common.Key");
    Reflector.setField(key, "key", "!!");
    final HashMap<String, String> properties = new HashMap<String, String>();
    properties.put("!!", "7");

    // Act
    final Long retval = TypedProperties.loong(key, properties);

    // result
    assertEquals(new Long(7L), retval);
  }

  // Test generated by Diffblue Deeptest.

  @Test
  public void loongInputNotNull1ZeroOutputPositive() throws Exception {

    // Arrange
    final Key<Long> key = (Key<Long>) Reflector.getInstance("com.kerio.im.common.Key");
    Reflector.setField(key, "key", null);
    final HashMap<String, String> properties = new HashMap<String, String>();
    properties.put(null, "+7");
    final Long defaultValue = 0L;

    // Act
    final Long retval = TypedProperties.loong(key, properties, defaultValue);

    // result
    assertEquals(new Long(7L), retval);
  }

  // Test generated by Diffblue Deeptest.

  @Test
  public void putAllInputNotNullOutputVoid() {

    // Arrange
    final TypedProperties objectUnderTest = new TypedProperties();
    final HashMap<Key, Object> hashMap = new HashMap<Key, Object>();
    Reflector.setField(objectUnderTest, "properties", hashMap);
    final TypedProperties from = new TypedProperties();
    final HashMap<Key, Object> hashMap1 = new HashMap<Key, Object>();
    hashMap1.put(null, 0);
    Reflector.setField(from, "properties", hashMap1);

    // Act
    objectUnderTest.putAll(from);
  }

  // Test generated by Diffblue Deeptest.

  @Test
  public void stringInputNotNull1NotNullOutputNull() throws Exception {

    // Arrange
    final Key<String> key = (Key<String>) Reflector.getInstance("com.kerio.im.common.Key");
    Reflector.setField(key, "key", null);
    final HashMap<String, String> properties = new HashMap<String, String>();
    properties.put(null, null);
    final String defaultValue = "";

    // Act
    final String retval = TypedProperties.string(key, properties, defaultValue);

    // result
    assertNull(retval);
  }

  // Test generated by Diffblue Deeptest.

  @Test
  public void stringInputNotNull1OutputIllegalArgumentException() throws Exception {

    // Arrange
    final Key<String> key = (Key<String>) Reflector.getInstance("com.kerio.im.common.Key");
    Reflector.setField(key, "key", "!  !");
    final HashMap<String, String> properties = new HashMap<String, String>();
    properties.put(null, null);

    // Act
    thrown.expect(IllegalArgumentException.class);
    TypedProperties.string(key, properties);

    // Method is not expected to return due to exception thrown
  }

  // Test generated by Diffblue Deeptest.

  @Test
  public void stringInputNotNull1OutputNotNull() throws Exception {

    // Arrange
    final Key<String> key = (Key<String>) Reflector.getInstance("com.kerio.im.common.Key");
    Reflector.setField(key, "key", null);
    final HashMap<String, String> properties = new HashMap<String, String>();
    properties.put(null, "    ");

    // Act
    final String retval = TypedProperties.string(key, properties);

    // result
    assertEquals("    ", retval);
  }
}
