/*
 * Copyright 2013 Kerio Technologies s.r.o. All rights reserved.
 */
package com.kerio.im.common;

import java.util.HashMap;
import java.util.Map;
import static org.junit.Assert.*;
import org.junit.Before;
import org.junit.Test;

/**
 *
 * @author tskoda
 */
public class CommandLineParserTest {
    
 private Map<String, String> properties;

    @Before
    public void beforeTest() {
        properties = new HashMap<String, String>();
    }

    /**
     * Test of load method, of class CommandLineCommandLineParser.
     */
    @Test
    public void testLoad() {
        String[] pairs = new String[2];

        pairs[0] = "--key";
        pairs[1] = "value";
        properties = CommandLineParser.toProperties(pairs);
        assertEquals("value", properties.get("key"));

        pairs = new String[1];
        pairs[0] = "key=value";
        properties = CommandLineParser.toProperties(pairs);
        assertEquals("value", properties.get("key"));

        pairs = new String[3];
        pairs[0] = "key1=value1";
        pairs[1] = "--key2";
        pairs[2] = "value2";
        properties = CommandLineParser.toProperties(pairs);
        assertEquals("value1", properties.get("key1"));
        assertEquals("value2", properties.get("key2"));


        pairs[0] = "--key2";
        pairs[1] = "value2";
        pairs[2] = "key1=value1";
        properties = CommandLineParser.toProperties(pairs);
        assertEquals("value1", properties.get("key1"));
        assertEquals("value2", properties.get("key2"));
    }

    /**
     * Test of load method, of class PropertiesUtil.
     */
    @Test(expected = IllegalArgumentException.class)
    public void testLoadUnknownKeyMissingPrefix() {
        String[] pairs = new String[1];

        pairs[0] = "-key value";
        CommandLineParser.toProperties(pairs);

    }

    /**
     * Test of load method, of class CommandLineCommandLineParser.
     */
    @Test(expected = IllegalArgumentException.class)
    public void testLoadNoDelimiter() {
        String[] pairs = new String[1];

        pairs[0] = "key value";
        CommandLineParser.toProperties(pairs);

    }

    /**
     * Test of load method, of class CommandLineCommandLineParser.
     */
    @Test(expected = IllegalArgumentException.class)
    public void testLoadUnknownDelimiter() {
        String[] pairs = new String[1];

        pairs[0] = "key-value";
        CommandLineParser.toProperties(pairs);

    }
}
