/*
 *  Copyright 2013 Kerio Technologies s.r.o. All rights reserved.
 */
package com.kerio.im.core.api.service;

import java.time.LocalDateTime;

/**
 * Defines methods called when content of conversation changes.
 */
public interface ConversationService {
    //TODO v.2 we also need to know when user JOINS room so we can display a history to him even if he did not say anything
    //TODO v.2 we also (probably) need to respect room setting to not store history at all

    /**
     * Defines type of message.
     */
    enum MessageType {

        /**
         * Message sent between two people in regular chat.
         */
        chat,
        /**
         * Message sent to a group.
         */
        group
    }

    /**
     * Provides file transfer details.
     */
    interface FileInfo {

        String name();

        long size();
    }

    /**
     * Provides sent message details
     */
    interface MessageInfo {

        String content();

        MessageType type();
    }

    /**
     * Called when message is sent.
     *
     * @param from jid of sender
     * @param to jid of receiver
     * @param info message info
     * @param time time when message has been sent
     * @param raw raw content of stanza
     */
    void message(String from, String to, MessageInfo info, LocalDateTime time, String raw);

    /**
     * Called when file is sent.
     *
     * @param from bare jid of sender
     * @param to bare jid of receiver
     * @param info file info
     * @param time time when file has been offered
     * @param raw raw content of stanza
     */
    void file(String from, String to, FileInfo info, LocalDateTime time, String raw);

}
