/*
 * Copyright 2013 Kerio Technologies s.r.o. All rights reserved.
 */
package com.kerio.im.core;

import com.kerio.im.core.api.service.ServiceBus;
import java.util.HashMap;
import java.util.LinkedList;
import java.util.List;
import java.util.Map;
import java.util.logging.Level;
import java.util.logging.Logger;

/**
 *
 * @author tskoda
 */
public class Bus implements ServiceBus {

    private static final Logger LOG = Logger.getLogger(Bus.class.getName());
    private final Map<Class, List<EventListener>> subscribers;

    public Bus() {
        this.subscribers = new HashMap<Class, List<EventListener>>();
    }

      //TODO add to event info, if its can be published single or multiple times
    @Override
    public synchronized <T extends Event> void publish(T event) {
        LOG.log(Level.FINEST, "Publishing event {0}", event);
        //one thread could call subscribe on some other event when processing another published event
        List<EventListener> listeners = new LinkedList<EventListener>(getListeners(event.getClass()));
        for (EventListener listener : listeners) {
            listener.event(event);
        }
    }

    @Override
    public synchronized <T extends Event> void publish(T event, ExceptionHandler handler) {
        LOG.log(Level.FINEST, "Publishing event {0}", event);
        //one thread could call subscribe on some other event when processing another published event
        List<EventListener> listeners = new LinkedList<EventListener>(getListeners(event.getClass()));
        for (EventListener listener : listeners) {
            try {
                listener.event(event);
            } catch (Exception e) {
                if (!handler.pass(e)) {
                    break;
                }
            }
        }
    }

    @Override
    public synchronized <T extends Event> void subscribe(Class<T> event, EventListener<T> listener) {
        LOG.log(Level.FINEST, "Addind subscription to event {0}", event);
        getListeners(event).add(listener);
    }
    
    private <T extends Event> List<EventListener> getListeners(Class<T> event) {
        List<EventListener> list = this.subscribers.get(event);
        if (list == null) {
            list = new LinkedList<EventListener>();
            this.subscribers.put(event, list);
        }
        return list;

    }
}
