/*
 * Copyright 2013 Kerio Technologies s.r.o. All rights reserved.
 */
package com.kerio.im.core;

import com.kerio.im.core.api.service.ServiceBus;
import com.kerio.im.core.auth.Authentication;
import com.kerio.im.core.configuration.StoreConfiguration;
import com.kerio.im.core.conversation.ConversationServiceImpl;
import com.kerio.im.core.logging.LoggingManager;
import com.kerio.im.core.remote.RemoteControl;
import com.kerio.im.core.service.SecurityManager;
import com.kerio.im.core.user.GroupProvider;
import com.kerio.im.core.user.UserAccessPolicy;
import java.util.Map;

/**
 *
 * @author tskoda
 */
public class CoreRuntimeFactory {
    
    private CoreRuntime coreRuntime = null;

    public CoreRuntimeFactory() {
    }

    /**
     * Return CoreService usable for other components. Method is not
     * thread-safe.
     *
     * @param properties command line properties
     * @return
     */
    public CoreRuntime get(Map<String, String> properties) {

        if (this.coreRuntime == null) {
            
            ServiceBus bus = new Bus();
            ThreadPool pool = new ThreadPool();
            Shutdown shutdown = new Shutdown(bus, pool);
            ErrorHandler errorHandler = new ErrorHandler(shutdown);
            LoggingManager manager = new LoggingManager(bus);
            StoreConfiguration storeConfiguration = new StoreConfiguration(properties, bus);
            ConversationServiceImpl conversations = new ConversationServiceImpl(bus);
            
            this.coreRuntime = new CoreRuntime(
                    manager,
                    storeConfiguration,
                    new Authentication(storeConfiguration, bus, errorHandler),
                    new RemoteControl(bus, errorHandler),
                    bus,
                    pool,
                    errorHandler,
                    new UserAccessPolicy(storeConfiguration),
                    conversations,
				    new GroupProvider(),
                    new SecurityManager());
        }

        return this.coreRuntime;
    }
}
