/*
 * Copyright 2013 Kerio Technologies s.r.o. All rights reserved.
 */
package com.kerio.im.core.auth;

import com.kerio.im.core.ErrorHandler;
import com.kerio.im.core.api.GlobalKey;
import com.kerio.im.core.api.service.AuthService;
import com.kerio.im.core.api.service.ServiceBus;
import com.kerio.im.core.configuration.StoreConfiguration;
import java.util.logging.Level;
import java.util.logging.Logger;

/**
 *
 * @author tskoda
 */
public class Authentication implements AuthService {

    private static final Logger LOG = Logger.getLogger(Authentication.class.getName());
    
    private final OfflineAuthentication offlineAuth;
    private final ServiceBus bus;
    private OnlineAuthentication onlineAuth;

    public Authentication(final StoreConfiguration configuration, ServiceBus bus, ErrorHandler errorHandler) {
        this.bus = bus;
        this.offlineAuth = new OfflineAuthentication(
                errorHandler,
                configuration.get(GlobalKey.SERVICE_REMOTE_LOGIN),
                configuration.get(GlobalKey.SERVICE_REMOTE_DOMAIN),
                configuration.get(GlobalKey.SERVICE_REMOTE_PORT));
    }

    public void set(final OnlineAuthentication onlineAuth) {
        this.onlineAuth = onlineAuth;
    }

    @Override
    public boolean auth(Credentials credentials, Connection connection) {
        LOG.log(Level.FINE, "Authenticating user {0}", credentials.getLogin());
        boolean result;
        
        
        if (this.onlineAuth != null) {
            result = this.onlineAuth.auth(credentials, connection);
        } else { //control user can authenticate only once, before onlineauth is set
            if (result = this.offlineAuth.auth(credentials, connection)) {
                this.bus.publish(new ControlAuthEvent());
            }
        }

        return result;
    }

    @Override
    public void disconnected(Credentials credentials) {
        LOG.log(Level.INFO, "Disconnecting user {0}", credentials.getLogin());
        //always try control user
        this.offlineAuth.disconnected(credentials);
        
        if (this.onlineAuth != null) {
            this.onlineAuth.disconnected(credentials);
        }
    }
}
