/*
 * Copyright 2013 Kerio Technologies s.r.o. All rights reserved.
 */
package com.kerio.im.core.backup;

import com.kerio.im.core.api.service.ServiceBus;
import com.kerio.im.core.remote.CommandEvent;
import com.kerio.im.xmpp.api.XmppDomains;

public class BackupListener implements ServiceBus.EventListener<CommandEvent>
{
    private static final String COMMAND = "backup";
    private static final String ContentPath = "backup.path";
    
    private final XmppDomains domains;
    
    
    public BackupListener(XmppDomains domains)
    {
        this.domains = domains;
    }
    
    public void subscribe(final ServiceBus bus)
    {
        bus.subscribe(CommandEvent.class, this);
    }
        
    @Override
    public void event(CommandEvent event)
    {
        if (!event.interested(COMMAND)) {
            return;
        }
        
        String backupPath = event.get(ContentPath);
        
        if (backupPath.isEmpty()) {
            return;
        }
        
        try {
            domains.backup(backupPath);
        }
        catch (Exception e) {
            event.fail(e.getMessage());
        }
    }
}