/*
 * Copyright 2013 Kerio Technologies s.r.o. All rights reserved.
 */
package com.kerio.im.core.cert;

import com.kerio.im.core.Bus;
import com.kerio.im.core.remote.CommandEvent;
import java.util.logging.Level;
import java.util.logging.Logger;

/**
 *
 * @author tskoda
 */
public class CertificateEventSync implements Bus.EventListener<CommandEvent> {

    private static final Logger LOG = Logger.getLogger(CertificateEventSync.class.getName());
    static final String COMMAND = "certificate";

    enum Operation {

        added,
        removed,
        changed;
    }

    enum Content {

        event("certificate.event"),
        id("certificate.id");
        
        final String namePath;

        private Content(String name) {
            this.namePath = name;
        }
    }
    
    private final Certificates certificates;

    public CertificateEventSync(final Certificates certificates) {
        this.certificates = certificates;
    }

    @Override
    public void event(CommandEvent event) {
        if (!event.interested(COMMAND)) {
            return;
        }

        try {
            Operation operation = Operation.valueOf(event.get(Content.event.namePath));
            String id = event.get(Content.id.namePath);
            if (id == null) {
                LOG.log(Level.WARNING, "Processing certificate event failed, certificate identifier is null");
                event.fail("Processing certificate event failed, certificate identifier is null");
            }
            switch (operation) {
                case added:
                    //certificate with provided id was added
                    this.certificates.addCertificate(id);
                    LOG.log(Level.INFO, "Adding new certificate successful");
                    break;
                case removed:
                    //certificate with provided id was removed
                    this.certificates.removeCertificate(id);
                    LOG.log(Level.INFO, "Removing certificate successful");
                    break;
                case changed:
                    //default certificate changed, id belongs to the new one
                    this.certificates.updateCertificate(id);
                    LOG.log(Level.INFO, "Changing default certificate successful");
                    break;
                default:
                    LOG.log(Level.WARNING, "Processing certificate event failed, unknown operation {0}", operation);
                    event.fail("Processing certificate event failed, unknown operation");
            }
        } catch (Exception e) {
            LOG.log(Level.WARNING, "Processing certificate event failed", e);
            event.fail("Processing certificate event failed");
        }
    }
}
