/*
 * Copyright 2014 Kerio Technologies s.r.o. All rights reserved
 */
package com.kerio.im.core.cert;

import com.kerio.im.bean.Certificate;
import java.util.HashMap;
import java.util.Map;
import java.util.Set;

/**
 *
 * @author tskoda
 */
public class CertificateStore {

    /**
     *
     */
    private final Map<String, Certificate> certificates;
    private final String defaultDomain;

    public CertificateStore(String defaultDomain) {
        this.certificates = new HashMap<>();
        this.defaultDomain = defaultDomain;
    }

    public void put(Certificate certificate, Set<String> names, boolean isDefault) {
        names.stream().forEach(name -> put(certificate, name));

        if (isDefault && (certificate.isValid() || certificates.get(defaultDomain) == null)) {
            certificates.put(defaultDomain, certificate);
        }
    }

    private boolean put(Certificate certificate, String name) {
        boolean result = false;
        if (certificate.isValid() || certificates.get(name) == null) {
            certificates.put(name, certificate);
            result = true;
        }
        return result;
    }


    public Map<String, Certificate> removeById(String id) {
        //find aliases for certificate
        //return it

        Map<String, Certificate> result = new HashMap<>();

        for (Map.Entry<String, Certificate> certEntry : certificates.entrySet()) {
            if (id.equals(certEntry.getValue().getId())) {
                result.put(certEntry.getKey(), null);
            }
        }

        certificates.keySet().removeAll(result.keySet());

        return result;
    }

    /**
     * Returns read only map.
     *
     * @return
     */
    public Map<String, Certificate> map() {
        return new HashMap<>(this.certificates);
    }

    public Map<String, Certificate> add(Certificate cert, Set<String> aliases) {
        Map<String, Certificate> result = new HashMap<>();

        for (String alias : aliases) {
            if (put(cert, alias)) {
                result.put(alias, cert);
            }
        }

        return result;
    }

    public Map<String, Certificate> updateDefault(String id) {
        Map<String, Certificate> result = new HashMap<>();
        
        for (Certificate cert : certificates.values()) {
            if (id.equals(cert.getId())) {
                result.put(defaultDomain, cert);
                certificates.put(defaultDomain, cert);
                break;
            }
        }
        
        return result;
    }

}
