/*
 *  Copyright 2013 Kerio Technologies s.r.o. All rights reserved.
 */
package com.kerio.im.core.conversation.archive;

import com.kerio.im.core.api.service.ConversationService;
import com.kerio.im.core.api.service.ServiceBus;
import com.kerio.im.core.conversation.ConversationServiceImpl;
import java.io.IOException;
import java.time.LocalDateTime;
import java.util.concurrent.ExecutorService;
import java.util.logging.Level;
import java.util.logging.Logger;

/**
 *
 * @author tskoda
 */
public class AsyncFileStore implements ConversationServiceImpl.AsyncConversationService {

    private static final Logger LOG = Logger.getLogger(AsyncFileStore.class.getName());

    private final ExecutorService executor;
    private final FileStore fileStore;
    private final ServiceBus.ExceptionHandler exceptionHandler;

    public AsyncFileStore(ExecutorService executor, FileStore fileStore, ServiceBus.ExceptionHandler exceptionHandler) {
        this.executor = executor;
        this.fileStore = fileStore;
        this.exceptionHandler = exceptionHandler;
    }

    @Override
    public ExecutorService getExecutor() {
        return this.executor;
    }

    @Override
    public void close() {
        this.executor.shutdownNow();
    }

    @Override
    public void message(String from, String to, ConversationService.MessageInfo info, LocalDateTime time, String raw) {
        LOG.log(Level.FINER, "Archiving {0} message from {1} to {2}", new Object[]{info.type(), from, to});
        try {
            this.fileStore.store(time, raw);
        } catch (IOException e) {
            this.exceptionHandler.pass(e);
        }
    }

    @Override
    public void file(String from, String to, ConversationService.FileInfo info, LocalDateTime time, String raw) {
        LOG.log(Level.FINER, "Archiving file transfer from {0} to {1}, file name {2}, size {3} kb", new Object[]{from, to, info.name(), info.size() / 1000});
        try {
            this.fileStore.store(time, raw);
        } catch (IOException e) {
            this.exceptionHandler.pass(e);
        }
    }
}
