/*
 * Copyright 2013 Kerio Technologies s.r.o. All rights reserved.
 */
package com.kerio.im.core.domain;

import com.kerio.im.bean.Domain;
import com.kerio.im.core.Bus;
import com.kerio.im.core.remote.CommandEvent;
import java.util.logging.Level;
import java.util.logging.Logger;

/**
 *
 * @author tskoda
 */
//TODO rename to EventSync
public class DomainEventSync implements Bus.EventListener<CommandEvent> {

    private static final String MESSAGE = "domain";
    private static final Logger LOG = Logger.getLogger(DomainEventSync.class.getName());

    private static enum OPERATION {

        created,
        removed,
        repair,
        restriction
    }

    private static enum PARAM {

        name("domain.name"),
        id("domain.id"),
        op("domain.event");
        public final String pname;

        private PARAM(String name) {
            this.pname = name;
        }
    }
    private final DomainControl control;

    public DomainEventSync(DomainControl control) {
        this.control = control;
    }

    @Override
    public void event(CommandEvent event) {
        if (!event.interested(MESSAGE)) {
            return;
        }
        try {
            OPERATION op = OPERATION.valueOf(event.get(PARAM.op.pname));
            Domain domain = new Domain(event.get(PARAM.id.pname), event.get(PARAM.name.pname).toLowerCase());
            synchronized (this.control) {
                switch (op) {
                    case created:
                        this.control.add(domain);
                        LOG.log(Level.INFO, "Processing domain event successful, domain {0} added", domain);
                        break;
                    case removed:
                        this.control.remove(domain);
                        LOG.log(Level.INFO, "Processing domain event successful, domain {0} removed", domain);
                        break;
                    case repair:
                        this.control.repair(domain);
                        LOG.log(Level.INFO, "Processing domain event successful, rosters of all users in domain {0} was repaired", domain);
                        break;
                    case restriction:
                        this.control.check(domain);
                        LOG.log(Level.INFO, "Processing domain event successful, domain {0} users restrictions updated", domain);
                        break;
                    default:
                        LOG.log(Level.WARNING, "Processing domain event failed, unsupported operation {0}", op);
                        event.fail("Processing domain message failed, unsupported operation");
                }
            }
        } catch (Exception e) {
            LOG.log(Level.WARNING, "Domain event processing failed", e);
            event.fail("Domain event processing failed");
        }
    }
}
