/*
 * Copyright 2013 Kerio Technologies s.r.o. All rights reserved.
 */
package com.kerio.im.core.logging;

import com.kerio.im.common.TypedProperties;
import java.util.logging.Filter;
import java.util.logging.Level;
import java.util.logging.LogRecord;

/**
 *
 * @author tskoda
 */
public class ConnectFilter implements Filter {
    
    private static final String CONNECT_ROOT = "com.kerio.im";
    private static final Level DEBUG_OFF = Level.WARNING;
    
    private final Level connectLevel;
    private final Level unknownLevel;
    private boolean debug;
    
    protected ConnectFilter(TypedProperties properties) {
        this.debug = properties.get(LogKeys.DEBUG);   
        this.connectLevel = properties.get(LogKeys.ENGINE_LEVEL);
        this.unknownLevel = connectLevel;   
    }

    @Override
    public boolean isLoggable(LogRecord lr) {
        String loggerName = lr.getLoggerName();
        
        Level level;
        
        if (!debug) {
            level = DEBUG_OFF;
        } else {
            level = isForeign(loggerName)? this.unknownLevel: this.connectLevel;
        }
        
        boolean result = lr.getLevel().intValue() >= level.intValue();
        return result;
    }
    
    protected boolean isForeign(String loggerName) {
        return !loggerName.startsWith(CONNECT_ROOT);
    }

    public void setDebug(boolean debug) {
        this.debug = debug;
    }
    
    
}
