/*
 * Copyright 2013 Kerio Technologies s.r.o. All rights reserved.
 */
package com.kerio.im.core.logging;

import java.io.ByteArrayOutputStream;
import java.io.PrintStream;
import java.text.SimpleDateFormat;
import java.util.logging.Formatter;
import java.util.logging.LogRecord;

/**
 *
 * @author development
 */
class FileFormatter extends Formatter {

    private final SimpleDateFormat sdf = new SimpleDateFormat("dd/MM/yyyy HH:mm:ss");
    private final StanzaAnonymizer stanzaAnonymizer;

    public FileFormatter() {
        this.stanzaAnonymizer = new StanzaAnonymizer();
    }

    @Override
    public String format(LogRecord record) {
        try {
            StringBuilder sb = new StringBuilder();
            sb.append("[");
            sb.append(record.getThreadID());
            sb.append("] ");

            sb.append("[");
            sb.append(sdf.format(record.getMillis()));
            sb.append("] ");
            sb.append(record.getLevel().getName());
            sb.append(" ");
            if (record.getMessage() != null) {
                Object[] params = record.getParameters();
                String message = record.getMessage();
                if (params != null) {
                    for (int i = 0; i < params.length; i++) {
                        String str = params[i] != null ? params[i].toString() : "null";
                        message = message.replace("{" + i + "}", str);
                    }
                }

                sb.append(this.stanzaAnonymizer.anonymize(record.getLoggerName(), message));
                //sb.append(message);
            }
            sb.append(" (");
            String className = record.getSourceClassName();
//            if (className.contains(".")) {
//                className = className.substring(className.lastIndexOf('.') + 1);
//            }
            sb.append(className);
            sb.append(".");
            sb.append(record.getSourceMethodName());
            sb.append(")");
            if (record.getThrown() != null) {
                sb.append(": ");
            }

            Throwable exception = record.getThrown();

            if ((exception == null) && (record.getParameters() != null) && (record.getParameters().length > 0)) {
                Object possiblyException = record.getParameters()[record.getParameters().length - 1];
                if (possiblyException instanceof Throwable) {
                    exception = (Throwable) possiblyException;
                }
            }

            if (exception != null) {
                ByteArrayOutputStream baos = new ByteArrayOutputStream();
                exception.printStackTrace(new PrintStream(baos));
                sb.append(new String(baos.toByteArray()));
            }
            sb.append("\n");
            return sb.toString();
        } catch (Exception e) {
            LoggingManager.STDERR_STREAM.println(e.getMessage());
        }
        return "";
    }
}
