/*
 * Copyright 2013 Kerio Technologies s.r.o. All rights reserved.
 */
package com.kerio.im.core.logging;

import com.kerio.im.common.FileUtil;
import com.kerio.im.common.Lookup;
import com.kerio.im.common.TypedProperties;
import com.kerio.im.core.api.CoreLogging;
import java.io.File;
import java.io.IOException;
import java.util.logging.Handler;
import java.util.logging.Level;
import java.util.logging.LogRecord;

/**
 *
 * @author tskoda
 */
public class FileHandler extends Handler {

    private static final int LIMIT = 10000000;
    private static final int COUNT = 5;
    private static final boolean APPEND = true;
    private static final String ENCODING = "UTF-8";
    /**
     * Default implementation of FileHandler
     */
    private final java.util.logging.FileHandler handlerImpl;
    

    public FileHandler() throws IOException {
        this(LoggingManager.getProperties());
    }

    protected FileHandler(TypedProperties properties) throws IOException {
        File file = properties.get(LogKeys.FILE);

        if (file != null) {
            file = FileUtil.createParent(properties.get(LogKeys.FILE));
            this.handlerImpl = new java.util.logging.FileHandler(file.getAbsolutePath(), LIMIT, COUNT, APPEND);
            this.handlerImpl.setLevel(Level.ALL);
            this.handlerImpl.setFormatter(new FileFormatter());
            this.handlerImpl.setEncoding(ENCODING);
            this.handlerImpl.setFilter(new FileFilter(properties));
        } else {
            this.handlerImpl = null;
        }
    }

    @Override
    public void publish(LogRecord record) {
        if (handlerImpl != null) {
            try {
                if (isLoggable(record)) {
                    this.handlerImpl.publish(record);
                }
            } catch (Exception e) {
                LoggingManager.STDERR_STREAM.println(e.getMessage());
            }
        }
    }

    @Override
    public void close() throws SecurityException {
        if (this.handlerImpl != null) {
            this.handlerImpl.close();
        }
    }

    @Override
    public void flush() {
        if (this.handlerImpl != null) {
            this.handlerImpl.flush();
        }
    }
}
