/*
 * Copyright 2013 Kerio Technologies s.r.o. All rights reserved.
 */
package com.kerio.im.core.remote;

import com.kerio.im.core.api.service.MessageService.ResponseHandler;
import com.kerio.im.core.api.service.ServiceBus;
import java.util.Map;

/**
 *
 * @author tskoda
 */
public class CommandEvent implements ServiceBus.Event {

    private final String type;
    private final String id;
    private final Map<String, String> content;
    private final Map<String, String> response;
    private final ResponseHandler responseHandler;
    private String failMessage;
    private boolean fail;
    private boolean async;
    
    public CommandEvent(String id, String type, Map<String, String> content, Map<String, String> response, ResponseHandler responseHandler) {
        this.id = id;
        this.type = type;
        this.content = content;
        this.response = response;
        this.fail = Boolean.FALSE;
        this.async = Boolean.FALSE;
        this.responseHandler = responseHandler;
    }
    
    public CommandEvent chain(String type, Map<String, String> content) {
        return new CommandEvent(this.id, type, content, this.response, this.responseHandler);
    }
    
    public void async() {
        this.async = Boolean.TRUE;
    }

    boolean isAsync() {
        return async;
    }

    Map<String, String> getResponse() {
        return response;
    }

    ResponseHandler getResponseHandler() {
        return responseHandler;
    }
    
    
    
    public String get(String name) {
        String result;
        if ((result = get(name, null)) == null) {
            throw new IllegalArgumentException("Required parameter " + name + " is missing");
        } 
        
        return result;
    }

    public String get(String name, String optional) {
        return content.containsKey(name)? content.get(name): optional;
    }
    
    public boolean contains(String name) {
        return get(name, null) != null;
    }

    public Map<String, String> getContent() {
        return content;
    }

    public String getType() {
        return type;
    }

    public String getId() {
        return id;
    }

    public boolean isFail() {
        return fail;
    }

    public String getFailMessage() {
        return failMessage;
    }

    public void fail(String message) {
        this.fail = Boolean.TRUE; //in case message passed from exception is null
        this.failMessage = message;
    }

    public void response(String key, String value) {
        if (!RemoteControl.PROTOCOL.result.is(key)) {
            this.response.put(key, value);
        }
    }

    public boolean interested(String type) {
        boolean result = Boolean.FALSE;

        if ((type != null) && (this.type != null)) {
            result = type.equalsIgnoreCase(this.type);
        }

        return result;
    }

    @Override
    public String toString() {
        return "CommandEvent{" + "type=" + type + ", id=" + id + ", content=" + content + '}';
    }
}
