/*
 * Copyright 2013 Kerio Technologies s.r.o. All rights reserved.
 */
package com.kerio.im.core.service;

import com.kerio.im.bean.Port;

/**
 *
 * @author tskoda
 */
public class ConnectionLimit implements Port.Limit {
    
    public static ConnectionLimit without() {
        return new ConnectionLimit(0);
    }
    
    public static ConnectionLimit one() {
        return new ConnectionLimit(1);
    }
    
    public static ConnectionLimit maximum(int maximum) {
        return new ConnectionLimit(maximum);
    }

    private int maximum;
    private long current;

    private ConnectionLimit(int maximum) {
        this.maximum = maximum;
        this.current = 0;
    }

    @Override
    public boolean pass() {
        //pass if maximum is not set (is zero) or is set and current is less then maximum
        synchronized (this) {
            return (this.maximum == 0) || (this.current < this.maximum);
        }

    }

    @Override
    public void connected() {
        synchronized(this) {
            this.current = Math.min(Long.MAX_VALUE, this.current + 1);
        }
    }

    @Override
    public void disconnected() {
        synchronized(this) {
            this.current = (Math.max(0, this.current - 1));
        }
    }
    
    public void update(int maximum) {
        synchronized(this) {
            this.maximum = maximum;
        }
    }

    public void update(ConnectionLimit limit) {
        synchronized(this) {
            this.maximum = limit.maximum;
        }
    }

    @Override
    public int getLimit() {
        return maximum;
    }
}
