/*
 *  Copyright 2013 Kerio Technologies s.r.o. All rights reserved.
 */
package com.kerio.im.core.service;

import com.kerio.im.bean.Port;
import com.kerio.im.external.ExtService;
import java.util.logging.Level;
import java.util.logging.Logger;

/**
 *
 * @author tskoda
 */
public class ConnectionListener implements Port.Listener {
    
    private static final Logger LOG = Logger.getLogger(ConnectionListener.class.getName());
    
    public static String asConnectinId(String ip, int port) {
        return ip + "_" + port;
    }
    
    public static ConnectionListener deaf() {
        return new ConnectionListener();
    }
    
    public static ConnectionListener external(ExtService extService) {
        return new ConnectionListener(extService);
    }
    
    private final ExtService extService; //can be null, see deaf()
    
    private ConnectionListener() {
        this.extService = null;
    }

    private ConnectionListener(ExtService extService) {
        this.extService = extService;
    }
    
    @Override
    public boolean connected(String serviceId, Boolean secure, String remoteAddress, int remotePort) {
        boolean result = true; //allow if no ext service provided
        
        LOG.log(Level.INFO, "Registering connection {0} to {1}service {2} from {3}:{4}", 
                new Object[]{asConnectinId(remoteAddress, remotePort), secure? "secure ": "", serviceId, remoteAddress, remotePort});
        
        if ((this.extService != null) && (serviceId != null) && (remoteAddress != null) && (remotePort > 0)) {
            try {
                this.extService.connect(serviceId, asConnectinId(remoteAddress, remotePort), remoteAddress, remotePort, secure);
            } catch (Exception e) {
                LOG.log(Level.SEVERE, "Registering connection failed", e);
                result = false;
            }    
        }
        
        return result;
    }

    @Override
    public void disconnected(String serviceId, String remoteAddress, int remotePort) {
        LOG.log(Level.INFO, "Unregistering connection {0} from {1}service {2} from {3}:{4}", 
                new Object[]{asConnectinId(remoteAddress, remotePort), serviceId, remoteAddress, remotePort});
        if ((this.extService != null) && (serviceId != null) && (remoteAddress != null) && (remotePort > 0)) {
            try {
                this.extService.disconnect(serviceId, asConnectinId(remoteAddress, remotePort));
            } catch (Exception e) {
                LOG.log(Level.SEVERE, "Unregistering connection failed", e);
            }
        }
    }
    
}
