/*
 * Copyright 2013 Kerio Technologies s.r.o. All rights reserved.
 */
package com.kerio.im.core.service;

import com.kerio.im.core.api.service.ServiceBus;
import com.kerio.im.core.remote.CommandEvent;
import java.util.HashMap;
import java.util.Map;
import java.util.logging.Level;
import java.util.logging.Logger;

/**
 *
 * @author tskoda
 */
public class EventGroup implements ServiceBus.EventListener<CommandEvent> {

    public static final String MESSAGE = "ipgroup";
    private static final Logger LOG = Logger.getLogger(EventGroup.class.getName());

    public enum Event {

        changed
    }

    public enum Content {

        event("ipgroup.event");
        public final String id;

        private Content(String id) {
            this.id = id;
        }
    }
    private final ServiceBus bus;
    private final ServiceProcessor processor;

    public EventGroup(ServiceBus bus, ServiceProcessor processor) {
        this.bus = bus;
        this.processor = processor;
    }

    @Override
    public void event(CommandEvent event) {
        if (!event.interested(MESSAGE)) {
            return;
        }

        try {
            Event operation = Event.valueOf(event.get(Content.event.id));
            switch (operation) {
                case changed:
                    LOG.log(Level.FINE, "Processing ipgroup event");
                    //publish change for all services
                    for (String id : this.processor.runningServices()) {
                        if ((!ServiceManager.SERVICE_CONTROL_ID.equals(id)) && (!EventFederation.FEDERATION_SERVICE_ID.equals(id))) {
                            Map<String, String> content = new HashMap<String, String>();
                            content.put(EventService.Content.action.id, EventService.Operation.changed.toString());
                            content.put(EventService.Content.identifier.id, id);
                            this.bus.publish(event.chain(EventService.MESSAGE, content)); //does not throw any exception
                        }
                    }

                    //publish change for global security settings
                    Map<String, String> content = new HashMap<String, String>();
                    content.put(EventService.Content.action.id, EventService.Operation.security.toString());
                    this.bus.publish(event.chain(EventService.MESSAGE, content));

                    if (event.isFail()) {
                        LOG.log(Level.WARNING, "Processing ipgroup event failed", event.getFailMessage());
                    } else {
                        LOG.log(Level.INFO, "Processing ipgroup event successful", event.getFailMessage());
                    }

                    break;

                default:
                    LOG.log(Level.WARNING, "Processing ipgroup event failed, no suitable operation found {0}", operation);
                    event.fail("Processing ipgroup event failed, no suitable operation found");

            }
        } catch (Exception e) {
            LOG.log(Level.WARNING, "Processing ipgroup event failed", e);
            event.fail(e.getMessage());
        }
    }
}
