/*
 * Copyright 2013 Kerio Technologies s.r.o. All rights reserved.
 */
package com.kerio.im.core.service;

import com.kerio.im.core.api.service.ServiceBus;
import com.kerio.im.core.remote.CommandEvent;
import java.util.logging.Level;
import java.util.logging.Logger;

/**
 *
 * @author tskoda
 */
public class EventStatus implements ServiceBus.EventListener<CommandEvent> {

    private static final Logger LOG = Logger.getLogger(EventStatus.class.getName());
    private static final String MESSAGE = "get";
    private static final String TYPE = "type";
    private static final String TYPE_VALUE = "federation";
    
    private final ServiceProcessor processor;
    private final String serviceId;

    public EventStatus(ServiceProcessor processor, String federationServiceId) {
        this.processor = processor;
        this.serviceId = federationServiceId;
    }
    
    @Override
    public void event(CommandEvent event) {
        if (!event.interested(MESSAGE)) {
            return;
        }

        try {
            String type = event.get(TYPE);

            if (TYPE_VALUE.equals(type)){
                switch(this.processor.status(serviceId)) {
                    case Error:
                        event.fail("Unknown error");
                        break;
                    case Running:
                    case Stopped:
                    case Unknown:
                    default:
                        //ok response is added by default
                }
            }
        } catch (Exception e) {
            LOG.log(Level.WARNING, "Processing federation settings event failed", e);
            event.fail(e.getMessage());
        }
    }
}
