/*
 * Copyright 2013 Kerio Technologies s.r.o. All rights reserved.
 */
package com.kerio.im.core.service;

import com.kerio.im.bean.Port;
import com.kerio.im.core.ThreadPool;
import com.kerio.im.core.api.GlobalKey;
import com.kerio.im.core.api.service.ConfigurationService;
import com.kerio.im.core.api.service.ServiceBus;
import com.kerio.im.core.domain.Domains;
import com.kerio.im.core.remote.CommandEvent;
import com.kerio.im.core.remote.RemoteControl;
import com.kerio.im.external.ExtService;
import com.kerio.im.xmpp.api.XmppPorts;
import java.util.logging.Level;
import java.util.logging.Logger;

/**
 *
 * @author tskoda
 */
public class ServiceManager {

    private static final Logger LOG = Logger.getLogger(ServiceManager.class.getName());
    private static final boolean SERVICE_TYPE_NOT_SSL = false;
    public static final Port.Security SERVICE_SECURITY_NOT_REQUIRED = Security.optional();
    public static final String ONLY_FROM_LOCALHOST = "localhost";
    public static final String SERVICE_NAME = "Instant Messaging Control";
    public static final String SERVICE_CONTROL_ID = "//control/" + ONLY_FROM_LOCALHOST;
    private final String hostname;
    private final ServiceProcessor serviceProcessor;

    public ServiceManager(ConfigurationService configuration, ServiceBus bus, XmppPorts xmppPorts) {
        this.hostname = configuration.get(GlobalKey.HOSTNAME);
        this.serviceProcessor = new ServiceProcessor(xmppPorts);
        
        ServicePort servicePort = createServicePort(configuration.get(GlobalKey.SERVICE_REMOTE_PORT));

        LOG.log(Level.FINE, "Starting control service {0}", servicePort.getId());
        this.serviceProcessor.start(servicePort.getId(), ServiceProcessor.asSet(servicePort), XmppPorts.Type.CLIENT);
        LOG.log(Level.INFO, "Starting control service on port {0} successful, listening on {1} only", new Object[]{servicePort.getPort(), ONLY_FROM_LOCALHOST});
        
        bus.subscribe(CommandEvent.class, new EventStatus(serviceProcessor, EventFederation.FEDERATION_SERVICE_ID));
    }

    public void set(ExtService extService, ServiceBus bus, ThreadPool pool, RemoteControl control, Domains domains) {
        this.serviceProcessor.setExtService(extService);
        bus.subscribe(
                CommandEvent.class,
                new EventService(extService, this.serviceProcessor, this.hostname));
        bus.subscribe(CommandEvent.class, 
                new EventFederation(extService, this.serviceProcessor, this.hostname));
        bus.subscribe(CommandEvent.class, 
                new EventPing(domains, bus, pool, control));
        bus.subscribe(CommandEvent.class, 
                new EventGroup(bus, this.serviceProcessor));
    }

    public String getServiceId(int port) {
        return this.serviceProcessor.getServiceId(port);
    }

    private ServicePort createServicePort(int servicePort) {
        return new ServicePort(
                SERVICE_CONTROL_ID,
                servicePort,
                ONLY_FROM_LOCALHOST,
                this.hostname,
                SERVICE_TYPE_NOT_SSL,
                SERVICE_SECURITY_NOT_REQUIRED,
                SERVICE_NAME,
                CombinedFilter.with(new FilterLocalhost()),
                ConnectionLimit.one(),
                ConnectionListener.deaf());
    }
}
