/*
 * Copyright 2013 Kerio Technologies s.r.o. All rights reserved.
 */
package com.kerio.im.core.user;

import com.kerio.im.core.Bus;
import com.kerio.im.core.domain.DomainEvent;
import java.util.logging.Level;
import java.util.logging.Logger;

/**
 *
 * @author tskoda
 */
public class EventDomainSync implements Bus.EventListener<DomainEvent> {

    private static final Logger LOG = Logger.getLogger(EventDomainSync.class.getName());
    private final UserControlList userControlList;

    public EventDomainSync(UserControlList userControlList) {
        this.userControlList = userControlList;
    }

    @Override
    public void event(DomainEvent event) {
        try {
            switch (event.type()) {
                case ADDED:
                    userControlList.create(event.domain());
                    break;
                case REMOVED:
                    userControlList.remove(event.domain());
                    break;
                case FOUND:
                    userControlList.sync(event.domain());
                    break;
                case REPAIRED:
                    userControlList.repair(event.domain());
                    break;
                default:
                    LOG.log(Level.WARNING, "Received unknown domain event {0}", event.type());
            }
            
            
        } catch (Exception e) {
            LOG.log(Level.INFO, "Domain event processing failed", e);
            LOG.log(Level.WARNING, "Domain event processing failed, {0}", e.getMessage());
        }
    }
}
