/*
 * Copyright 2013 Kerio Technologies s.r.o. All rights reserved.
 */
package com.kerio.im.core.user;

import com.kerio.im.bean.Domain;
import com.kerio.im.bean.User;
import com.kerio.im.external.ExtService;
import java.util.Collection;
import java.util.HashMap;
import java.util.HashSet;
import java.util.Map;
import java.util.logging.Level;
import java.util.logging.Logger;

/**
 *
 * @author tskoda
 */
public class UserCache {
    
    private static final Logger LOG = Logger.getLogger(UserCache.class.getName());

    private static final int AVERAGE_NUMBER_OF_USERS = 50;
    private final Domain domain;
    private final ExtService extService;
    private final Map<String, User> users;

    public UserCache(Domain domain, ExtService extService) {
        this.domain = domain;
        this.extService = extService;
        this.users = new HashMap<String, User>(AVERAGE_NUMBER_OF_USERS);
    }

    public void refresh() {
        LOG.log(Level.FINE, "Loading all users from domain {0} from external service", this.domain.getName());
        Map<String, User> newUsers = new HashMap<String, User>();
        //this fails with service exception in case LDAP source is not accessible, cached users remains intact
        this.extService.getAllUsers(this.domain.getId(), new ExternalUser(this.domain.getName(), newUsers));
        this.users.clear();
        this.users.putAll(newUsers);
        LOG.log(Level.FINE, "Loading all users from domain {0} from external service successful", this.domain.getName());
        
    }
    
    public void addOrUpdate(User user) {
        this.users.put(user.getLogin(), user);
    }
    
    public void remove(User user) {
        this.users.remove(user.getLogin());
    }

    public User find(final String userName) {
        return this.users.get(userName);
    }

    public Collection<User> all(User except) {
        Collection<User> result = new HashSet<User>(this.users.values());
        result.remove(except);
        return result;
    }

    public Collection<User> all() {
        Collection<User> result = new HashSet<User>(this.users.values());
        return result;
    }

    protected static class ExternalUser implements ExtService.UserHandler {
        
        final String domain;
        final Map<String, User> users;

        public ExternalUser(String domain, Map<String, User> users) {
            this.domain = domain;
            this.users = users;
        }
        
        @Override
        public void handle(String login, String fullname, boolean enabled, boolean inGal, boolean domainRestriction ) {

            User user = new User(login.toLowerCase(), this.domain, fullname, enabled, inGal, domainRestriction);
            this.users.put(login, user);

        }
    }
}
