/*
 * Copyright 2013 Kerio Technologies s.r.o. All rights reserved.
 */
package com.kerio.im.core.vcard;

import com.kerio.im.core.Bus;
import com.kerio.im.core.domain.DomainEvent;
import java.util.logging.Level;
import java.util.logging.Logger;

/**
 *
 * @author tskoda
 */
public class EventDomainSync implements Bus.EventListener<DomainEvent>  {
    
    private static final Logger LOG = Logger.getLogger(EventDomainSync.class.getName());
    private final VCardStore vcardStore;

    public EventDomainSync(VCardStore vcardStore) {
        this.vcardStore = vcardStore;
    }
    
    @Override
    public void event(DomainEvent event) {
        try {
            switch (event.type()) {
                case ADDED:
                    this.vcardStore.add(event.domain().getName());
                    //TODO move first full refresh on first vcard download
                    this.vcardStore.refresh(event.domain().getName());
                    break;
                case REMOVED:
                    this.vcardStore.remove(event.domain().getName());
                    break;
                case FOUND:
                    this.vcardStore.refresh(event.domain().getName());
                    break;
                default:
                    LOG.log(Level.WARNING, "Received unknown domain event {0}", event.type());
            }
            
            
        } catch (Exception e) {
            LOG.log(Level.WARNING, "Domain event processing failed, cannot process vcards", e);
        }
    }
    
}
