/*
 * Copyright 2013 Kerio Technologies s.r.o. All rights reserved.
 */
package com.kerio.im.core.vcard;

import com.kerio.im.bean.User;
import com.kerio.im.bean.VCard;
import com.kerio.im.bean.VCard.VCardFactory;
import com.kerio.im.external.ExtService;
import java.util.HashSet;
import java.util.Set;
import java.util.concurrent.ConcurrentHashMap;
import java.util.logging.Level;
import java.util.logging.Logger;

/**
 *
 * @author tskoda
 */
public class VCardStore  {
    
    private static final Logger LOG = Logger.getLogger(VCardStore.class.getName());

    private final VCardFactory factory;
    private final ConcurrentHashMap<String, VCardCache> cache;
    private final ExtService extService;
    private boolean enabled;

    public VCardStore(VCardFactory factory, ExtService extService) {
        this.factory = factory;
        this.extService = extService;
        this.cache = new ConcurrentHashMap<String, VCardCache>();
    }
    
    public VCard get(String login, String domain) {
        VCardCache domainCache = get(domain);
        VCard result = null;
        if (domainCache != null) {
            result = domainCache.get(login);
        } else {
            LOG.log(Level.INFO, "Getting vcard for user {0}@{1} failed, vcard cache for domain {1} does not exist", new Object[]{login, domain});
        }
        
        return result;
    }
    
    public void add(String domain) {
        this.cache.putIfAbsent(domain, new VCardCache(domain));
    }
    
    public void remove(String domain) {
        this.cache.remove(domain);
    }
    
    public VCardCache get(String domain) {
        return cache.get(domain);
    }
    
    public void refresh(String domain) {
        VCardCache domainCache = get(domain);
        if (domainCache != null) {
            domainCache.refresh(this.extService, this.factory);
        }
    }
    
    public void refresh() {
        Set<String> domainsSet;
        
        //do not block all domains when refreshing
        synchronized(this.cache) {
            domainsSet = new HashSet(this.cache.keySet());
        }
        
        for (String domain: domainsSet) {
            refresh(domain);
        }
    }

}
