package com.kerio.im.core.auth;

import static org.junit.Assert.assertEquals;

import com.diffblue.deeptestutils.Reflector;
import com.diffblue.deeptestutils.mock.DTUMemberMatcher;
import com.kerio.im.core.api.service.AuthService.Connection;
import com.kerio.im.core.api.service.AuthService.Credentials;
import java.lang.reflect.Method;
import org.junit.Rule;
import org.junit.Test;
import org.junit.rules.ExpectedException;
import org.junit.rules.Timeout;
import org.junit.runner.RunWith;
import org.powermock.api.mockito.PowerMockito;
import org.powermock.core.classloader.annotations.PrepareForTest;
import org.powermock.modules.junit4.PowerMockRunner;

@RunWith(PowerMockRunner.class)
public class OfflineAuthenticationDiffblueTest {

  @Rule public final ExpectedException thrown = ExpectedException.none();
  @Rule public final Timeout globalTimeout = new Timeout(10000);

  /* testedClasses: OfflineAuthentication */
  // Test generated by Diffblue Deeptest.
  @PrepareForTest(
      fullyQualifiedNames = {
        "com.kerio.im.core.api.service.AuthService$Connection",
        "com.kerio.im.core.api.service.AuthService$Credentials"
      })
  @Test
  public void authInputNotNullNotNullOutputFalse() throws Exception {

    // Arrange
    final OfflineAuthentication objectUnderTest =
        (OfflineAuthentication)
            Reflector.getInstance("com.kerio.im.core.auth.OfflineAuthentication");
    Reflector.setField(objectUnderTest, "errorHandler", null);
    Reflector.setField(objectUnderTest, "domain", "!!!!!!!!!");
    Reflector.setField(objectUnderTest, "port", 0);
    Reflector.setField(objectUnderTest, "login", "#######\"");
    final Credentials credentials = PowerMockito.mock(Credentials.class);
    final Method getUsernameMethod = DTUMemberMatcher.method(Credentials.class, "getUsername");
    PowerMockito.doReturn("\"\"\"\"\"\"\"#").when(credentials, getUsernameMethod).withNoArguments();
    final Connection connection = PowerMockito.mock(Connection.class);
    final Method getLocalPortMethod = DTUMemberMatcher.method(Connection.class, "getLocalPort");
    PowerMockito.doReturn(0).when(connection, getLocalPortMethod).withNoArguments();

    // Act
    final boolean retval = objectUnderTest.auth(credentials, connection);

    // result
    assertEquals(false, retval);
  }

  // Test generated by Diffblue Deeptest.
  @PrepareForTest(
      fullyQualifiedNames = {
        "com.kerio.im.core.api.service.AuthService$Connection",
        "com.kerio.im.core.api.service.AuthService$Credentials"
      })
  @Test
  public void authInputNotNullNotNullOutputTrue() throws Exception {

    // Arrange
    final OfflineAuthentication objectUnderTest =
        (OfflineAuthentication)
            Reflector.getInstance("com.kerio.im.core.auth.OfflineAuthentication");
    Reflector.setField(objectUnderTest, "errorHandler", null);
    Reflector.setField(objectUnderTest, "domain", "??????????");
    Reflector.setField(objectUnderTest, "port", 0);
    Reflector.setField(objectUnderTest, "login", "");
    final Credentials credentials = PowerMockito.mock(Credentials.class);
    final Method getUsernameMethod = DTUMemberMatcher.method(Credentials.class, "getUsername");
    PowerMockito.doReturn("").when(credentials, getUsernameMethod).withNoArguments();
    final Method getDomainMethod = DTUMemberMatcher.method(Credentials.class, "getDomain");
    PowerMockito.doReturn("??????????").when(credentials, getDomainMethod).withNoArguments();
    final Connection connection = PowerMockito.mock(Connection.class);
    final Method getLocalPortMethod = DTUMemberMatcher.method(Connection.class, "getLocalPort");
    PowerMockito.doReturn(0).when(connection, getLocalPortMethod).withNoArguments();

    // Act
    final boolean retval = objectUnderTest.auth(credentials, connection);

    // result
    assertEquals(true, retval);
  }

  // Test generated by Diffblue Deeptest.
  @PrepareForTest(fullyQualifiedNames = {"com.kerio.im.core.api.service.AuthService$Credentials"})
  @Test
  public void isServiceUserInputNotNullOutputFalse() throws Exception {

    // Arrange
    final OfflineAuthentication objectUnderTest =
        (OfflineAuthentication)
            Reflector.getInstance("com.kerio.im.core.auth.OfflineAuthentication");
    Reflector.setField(objectUnderTest, "errorHandler", null);
    Reflector.setField(objectUnderTest, "domain", null);
    Reflector.setField(objectUnderTest, "port", 0);
    Reflector.setField(objectUnderTest, "login", "");
    final Credentials credentials = PowerMockito.mock(Credentials.class);
    final Method getUsernameMethod = DTUMemberMatcher.method(Credentials.class, "getUsername");
    PowerMockito.doReturn("!").when(credentials, getUsernameMethod).withNoArguments();

    // Act
    final Class<?> classUnderTest =
        Reflector.forName("com.kerio.im.core.auth.OfflineAuthentication");
    final Method methodUnderTest =
        classUnderTest.getDeclaredMethod(
            "isServiceUser",
            Reflector.forName("com.kerio.im.core.api.service.AuthService$Credentials"));
    methodUnderTest.setAccessible(true);
    final boolean retval = (boolean) methodUnderTest.invoke(objectUnderTest, credentials);

    // result
    assertEquals(false, retval);
  }

  // Test generated by Diffblue Deeptest.
  @PrepareForTest(fullyQualifiedNames = {"com.kerio.im.core.api.service.AuthService$Credentials"})
  @Test
  public void isServiceUserInputNotNullOutputFalse2() throws Exception {

    // Arrange
    final OfflineAuthentication objectUnderTest =
        (OfflineAuthentication)
            Reflector.getInstance("com.kerio.im.core.auth.OfflineAuthentication");
    Reflector.setField(objectUnderTest, "errorHandler", null);
    Reflector.setField(objectUnderTest, "domain", "");
    Reflector.setField(objectUnderTest, "port", 0);
    Reflector.setField(objectUnderTest, "login", "   ");
    final Credentials credentials = PowerMockito.mock(Credentials.class);
    final Method getUsernameMethod = DTUMemberMatcher.method(Credentials.class, "getUsername");
    PowerMockito.doReturn("   ").when(credentials, getUsernameMethod).withNoArguments();
    final Method getDomainMethod = DTUMemberMatcher.method(Credentials.class, "getDomain");
    PowerMockito.doReturn("\"").when(credentials, getDomainMethod).withNoArguments();

    // Act
    final Class<?> classUnderTest =
        Reflector.forName("com.kerio.im.core.auth.OfflineAuthentication");
    final Method methodUnderTest =
        classUnderTest.getDeclaredMethod(
            "isServiceUser",
            Reflector.forName("com.kerio.im.core.api.service.AuthService$Credentials"));
    methodUnderTest.setAccessible(true);
    final boolean retval = (boolean) methodUnderTest.invoke(objectUnderTest, credentials);

    // result
    assertEquals(false, retval);
  }

  // Test generated by Diffblue Deeptest.
  @PrepareForTest(fullyQualifiedNames = {"com.kerio.im.core.api.service.AuthService$Credentials"})
  @Test
  public void isServiceUserInputNotNullOutputTrue() throws Exception {

    // Arrange
    final OfflineAuthentication objectUnderTest =
        (OfflineAuthentication)
            Reflector.getInstance("com.kerio.im.core.auth.OfflineAuthentication");
    Reflector.setField(objectUnderTest, "errorHandler", null);
    Reflector.setField(objectUnderTest, "domain", "\"");
    Reflector.setField(objectUnderTest, "port", 0);
    Reflector.setField(objectUnderTest, "login", "   ");
    final Credentials credentials = PowerMockito.mock(Credentials.class);
    final Method getUsernameMethod = DTUMemberMatcher.method(Credentials.class, "getUsername");
    PowerMockito.doReturn("   ").when(credentials, getUsernameMethod).withNoArguments();
    final Method getDomainMethod = DTUMemberMatcher.method(Credentials.class, "getDomain");
    PowerMockito.doReturn("\"").when(credentials, getDomainMethod).withNoArguments();

    // Act
    final Class<?> classUnderTest =
        Reflector.forName("com.kerio.im.core.auth.OfflineAuthentication");
    final Method methodUnderTest =
        classUnderTest.getDeclaredMethod(
            "isServiceUser",
            Reflector.forName("com.kerio.im.core.api.service.AuthService$Credentials"));
    methodUnderTest.setAccessible(true);
    final boolean retval = (boolean) methodUnderTest.invoke(objectUnderTest, credentials);

    // result
    assertEquals(true, retval);
  }

  // Test generated by Diffblue Deeptest.
  @PrepareForTest(fullyQualifiedNames = {"com.kerio.im.core.api.service.AuthService$Credentials"})
  @Test
  public void isServiceUserInputNotNullZeroOutputFalse() throws Exception {

    // Arrange
    final OfflineAuthentication objectUnderTest =
        (OfflineAuthentication)
            Reflector.getInstance("com.kerio.im.core.auth.OfflineAuthentication");
    Reflector.setField(objectUnderTest, "errorHandler", null);
    Reflector.setField(objectUnderTest, "domain", null);
    Reflector.setField(objectUnderTest, "port", 0);
    Reflector.setField(objectUnderTest, "login", "??????????");
    final Credentials credentials = PowerMockito.mock(Credentials.class);
    final Method getUsernameMethod = DTUMemberMatcher.method(Credentials.class, "getUsername");
    PowerMockito.doReturn("????????").when(credentials, getUsernameMethod).withNoArguments();
    final int port = 0;

    // Act
    final Class<?> classUnderTest =
        Reflector.forName("com.kerio.im.core.auth.OfflineAuthentication");
    final Method methodUnderTest =
        classUnderTest.getDeclaredMethod(
            "isServiceUser",
            Reflector.forName("com.kerio.im.core.api.service.AuthService$Credentials"),
            Reflector.forName("int"));
    methodUnderTest.setAccessible(true);
    final boolean retval = (boolean) methodUnderTest.invoke(objectUnderTest, credentials, port);

    // result
    assertEquals(false, retval);
  }

  // Test generated by Diffblue Deeptest.
  @PrepareForTest(fullyQualifiedNames = {"com.kerio.im.core.api.service.AuthService$Credentials"})
  @Test
  public void isServiceUserInputNotNullZeroOutputTrue() throws Exception {

    // Arrange
    final OfflineAuthentication objectUnderTest =
        (OfflineAuthentication)
            Reflector.getInstance("com.kerio.im.core.auth.OfflineAuthentication");
    Reflector.setField(objectUnderTest, "errorHandler", null);
    Reflector.setField(objectUnderTest, "domain", "!!!!!");
    Reflector.setField(objectUnderTest, "port", 0);
    Reflector.setField(objectUnderTest, "login", "         !");
    final Credentials credentials = PowerMockito.mock(Credentials.class);
    final Method getUsernameMethod = DTUMemberMatcher.method(Credentials.class, "getUsername");
    PowerMockito.doReturn("         !").when(credentials, getUsernameMethod).withNoArguments();
    final Method getDomainMethod = DTUMemberMatcher.method(Credentials.class, "getDomain");
    PowerMockito.doReturn("!!!!!").when(credentials, getDomainMethod).withNoArguments();
    final int port = 0;

    // Act
    final Class<?> classUnderTest =
        Reflector.forName("com.kerio.im.core.auth.OfflineAuthentication");
    final Method methodUnderTest =
        classUnderTest.getDeclaredMethod(
            "isServiceUser",
            Reflector.forName("com.kerio.im.core.api.service.AuthService$Credentials"),
            Reflector.forName("int"));
    methodUnderTest.setAccessible(true);
    final boolean retval = (boolean) methodUnderTest.invoke(objectUnderTest, credentials, port);

    // result
    assertEquals(true, retval);
  }
}
