package com.kerio.im.core.conversation.archive;

import static org.junit.Assert.assertEquals;
import static org.mockito.AdditionalMatchers.or;
import static org.mockito.Matchers.isA;
import static org.mockito.Matchers.isNull;

import com.diffblue.deeptestutils.Reflector;
import java.io.File;
import java.io.IOException;
import java.lang.reflect.Method;
import org.junit.Rule;
import org.junit.Test;
import org.junit.rules.ExpectedException;
import org.junit.rules.Timeout;
import org.junit.runner.RunWith;
import org.powermock.api.mockito.PowerMockito;
import org.powermock.core.classloader.annotations.PrepareForTest;
import org.powermock.modules.junit4.PowerMockRunner;

@RunWith(PowerMockRunner.class)
public class FileFactoryDiffblueTest {

  @Rule public final ExpectedException thrown = ExpectedException.none();
  @Rule public final Timeout globalTimeout = new Timeout(10000);

  /* testedClasses: FileFactory */
  // Test generated by Diffblue Deeptest.
  @PrepareForTest(File.class)
  @Test
  public void getDestinationOutputNotNull() throws Exception {

    // Arrange
    final FileFactory objectUnderTest =
        (FileFactory) Reflector.getInstance("com.kerio.im.core.conversation.archive.FileFactory");
    final File file = PowerMockito.mock(File.class);
    PowerMockito.when(file.getAbsolutePath()).thenReturn("");
    Reflector.setField(file, "prefixLength", 0);
    Reflector.setField(file, "path", "");
    Reflector.setField(objectUnderTest, "parent", file);

    // Act
    final String retval = objectUnderTest.getDestination();

    // result
    assertEquals("", retval);
  }

  // Test generated by Diffblue Deeptest.

  @Test
  public void removeSuffixInputNotNullOutputNotNull() throws Exception {

    // Arrange
    final FileFactory objectUnderTest =
        (FileFactory) Reflector.getInstance("com.kerio.im.core.conversation.archive.FileFactory");
    final File file = PowerMockito.mock(File.class);
    Reflector.setField(file, "prefixLength", 0);
    Reflector.setField(file, "path", null);
    Reflector.setField(objectUnderTest, "parent", file);
    final String filename = ".!!!!!!!";

    // Act
    final Class<?> classUnderTest =
        Reflector.forName("com.kerio.im.core.conversation.archive.FileFactory");
    final Method methodUnderTest =
        classUnderTest.getDeclaredMethod("removeSuffix", Reflector.forName("java.lang.String"));
    methodUnderTest.setAccessible(true);
    final String retval = (String) methodUnderTest.invoke(objectUnderTest, filename);

    // result
    assertEquals(".!!!!!!!", retval);
  }

  // Test generated by Diffblue Deeptest.
  @PrepareForTest(File.class)
  @Test
  public void zipInputNotNullNotNullOutputNullPointerException() throws Exception {

    // Arrange
    final FileFactory objectUnderTest =
        (FileFactory) Reflector.getInstance("com.kerio.im.core.conversation.archive.FileFactory");
    Reflector.setField(objectUnderTest, "parent", null);
    final File source = PowerMockito.mock(File.class);
    final NullPointerException nullPointerException =
        (NullPointerException) Reflector.getInstance("java.lang.NullPointerException");
    Reflector.setField(nullPointerException, "cause", null);
    Reflector.setField(nullPointerException, "detailMessage", null);
    PowerMockito.when(source.getName()).thenThrow(nullPointerException);
    PowerMockito.when(source.exists()).thenReturn(true);
    Reflector.setField(source, "prefixLength", 0);
    Reflector.setField(source, "path", null);
    final File destination = PowerMockito.mock(File.class);
    PowerMockito.when(destination.exists()).thenReturn(false);
    Reflector.setField(destination, "prefixLength", 0);
    Reflector.setField(destination, "path", "");

    // Act
    thrown.expect(NullPointerException.class);
    objectUnderTest.zip(source, destination);

    // Method is not expected to return due to exception thrown
  }

  // Test generated by Diffblue Deeptest.
  @PrepareForTest({File.class, FileFactory.class})
  @Test
  public void zipInputNotNullOutputIOException() throws Exception {

    // Arrange
    final FileFactory objectUnderTest =
        (FileFactory) Reflector.getInstance("com.kerio.im.core.conversation.archive.FileFactory");
    final File file = PowerMockito.mock(File.class);
    Reflector.setField(file, "prefixLength", 0);
    Reflector.setField(file, "path", null);
    Reflector.setField(objectUnderTest, "parent", file);
    final File source = PowerMockito.mock(File.class);
    PowerMockito.when(source.getName()).thenReturn("-????").thenReturn("-????");
    Reflector.setField(source, "prefixLength", 0);
    Reflector.setField(source, "path", "-????");
    final File file2 = PowerMockito.mock(File.class);
    PowerMockito.when(file2.getAbsolutePath()).thenReturn("    ");
    PowerMockito.when(file2.exists()).thenReturn(true);
    PowerMockito.when(file2.delete()).thenReturn(false);
    Reflector.setField(file2, "prefixLength", 0);
    Reflector.setField(file2, "path", "-????.part");
    final File file1 = PowerMockito.mock(File.class);
    PowerMockito.when(file1.exists()).thenReturn(false);
    Reflector.setField(file1, "prefixLength", 0);
    Reflector.setField(file1, "path", "-????.zip");
    PowerMockito.whenNew(File.class)
        .withParameterTypes(File.class, String.class)
        .withArguments(
            or(isA(File.class), isNull(File.class)), or(isA(String.class), isNull(String.class)))
        .thenReturn(file1)
        .thenReturn(file2);

    // Act
    thrown.expect(IOException.class);
    objectUnderTest.zip(source);

    // Method is not expected to return due to exception thrown
  }
}
