/*
 *  Copyright 2013 Kerio Technologies s.r.o. All rights reserved.
 */
package com.kerio.im.core.conversation;

import java.io.File;
import java.io.FileFilter;
import java.sql.Connection;
import java.sql.ResultSet;
import java.sql.SQLException;
import java.sql.Statement;
import java.util.Arrays;
import java.util.LinkedList;
import java.util.logging.Level;
import java.util.logging.Logger;
import org.apache.derby.jdbc.EmbeddedDataSource40;

/**
 *
 * @author tskoda
 */
public class TempDerby {

    private static final Logger LOG = Logger.getLogger(TempDerby.class.getName());

    public static void shutdown(EmbeddedDataSource40 ds) {

        try {
            ds.getConnection();
            System.err.println("Can not destroy test database: " + ds.toString());
        } catch (SQLException ex) {
            String engineShutdownSucc = "XJ015";
            String databaseShutdownSucc = "08006";
            if (engineShutdownSucc.equals(ex.getSQLState()) || (databaseShutdownSucc.equals(ex.getSQLState()))) {
                //ok
            } else {
                System.err.println("Can not destroy test database: " + ds.toString());
            }
        } catch (Exception ex) {
            System.err.println("Can not destroy test database: " + ds.toString());
        }

    }

    public static void close(ResultSet rs) {
        if (rs != null) {
            try {
                rs.close();
            } catch (Exception e) {
                System.err.println("Can not close Statement");
            }
        }
    }

    public static void close(Statement statement) {
        if (statement != null) {
            try {
                statement.close();
            } catch (Exception e) {
                System.err.println("Can not close Statement");
            }
        }
    }

    public static void close(Connection connection) {
        if (connection != null) {
            try {
                connection.close();
            } catch (Exception e) {
                System.err.println("Can not close connection");
            }
        }
    }

    private final TempFolder parentFolder;
    private final LinkedList<File> dbs;

    public TempDerby(TempFolder parentFolder) {
        this.parentFolder = parentFolder;
        this.dbs = new LinkedList<File>();
    }

    public File create(String name) throws SQLException {
        File db = this.parentFolder.get(name);
        this.dbs.addFirst(db);
        LOG.log(Level.INFO, "Created database {0}", db.getAbsolutePath());

        String dbName = db.getAbsolutePath();
        EmbeddedDataSource40 toCreate = new EmbeddedDataSource40();
        toCreate.setDatabaseName(dbName);
        toCreate.setCreateDatabase("create");

        close(toCreate.getConnection());

        shutdown(dbName);
        return db;
    }

    public void destroy() {
        for (File db : this.dbs) {
            destroyDerby(db);
        }
    }

    private void shutdown(String dbName) {
        EmbeddedDataSource40 ds = new EmbeddedDataSource40();
        ds.setDatabaseName(dbName);
        ds.setShutdownDatabase("shutdown");

        shutdown(ds);
    }

    private void destroyDerby(final File derbyFolder) {
        deleteFolderWithFiles(new File(derbyFolder, "log"));
        deleteFolderWithFiles(new File(derbyFolder, "seg0"));
        deleteFolderWithFiles(new File(derbyFolder, "tmp"));
        deleteFolderWithFiles(derbyFolder);
    }

    private void deleteFolderWithFiles(final File tempFolder) {
        if ((tempFolder == null) || (!tempFolder.exists()) || (!tempFolder.isDirectory())) {
            return;
        }

        File[] subfolders = tempFolder.listFiles(new FileFilter() {
            @Override
            public boolean accept(File pathname) {
                return pathname.isDirectory();
            }
        });

        if (subfolders.length > 0) {
            LOG.log(Level.WARNING, "Not going to delete folder with unexpected subfolders, subfolders: {0}", Arrays.toString(subfolders));
        }

        /**/
        for (File child : tempFolder.listFiles()) {
            if (!child.delete()) {
                LOG.log(Level.WARNING, "File {0} can't be deleted", child.getAbsoluteFile());
            } else {
                LOG.log(Level.INFO, "Deleted {0}", child.getAbsoluteFile());
            }
        }

        if (!tempFolder.delete()) {
            LOG.log(Level.WARNING, "Folder {0} can't be deleted", tempFolder.getAbsoluteFile());
        } else {
            LOG.log(Level.INFO, "Deleted {0}", tempFolder.getAbsoluteFile());
        }

    }

}
