/*
 * Copyright 2013 Kerio Technologies s.r.o. All rights reserved.
 */
package com.kerio.im.core.conversation;

import java.io.File;
import java.io.IOException;
import java.util.LinkedList;
import java.util.logging.Level;
import java.util.logging.Logger;
import org.apache.derby.impl.sql.execute.JarUtil;

/**
 *
 * @author tskoda
 */
public class TempFolder {

    private static final Logger LOG = Logger.getLogger(TempFolder.class.getName());

    private static File temp(File parent) throws IOException {
        File temp;

        temp = File.createTempFile("db_", Long.toString(System.nanoTime()), parent);

        if (!(temp.delete())) {
            throw new IOException("Could not delete temp file: " + temp.getAbsolutePath());
        }

        if (!(temp.mkdir())) {
            throw new IOException("Could not create temp directory: " + temp.getAbsolutePath());
        }

        return temp;
    }

    private final File tmpFolder;
    private final LinkedList<File> created;

    public TempFolder() throws IOException {
        this.created = new LinkedList<File>();
        this.tmpFolder = temp(null);
        this.created.addFirst(this.tmpFolder);
        
    }

    public void destroy() {
        for (File file : this.created) {
            if ((file != null) && (file.exists())){
                if (!file.delete()) {
                    LOG.log(Level.WARNING, "File {0} can't be deleted", file.getAbsoluteFile());
                } else {
                    LOG.log(Level.INFO, "Deleted {0}", file.getAbsoluteFile());
                }
            } else {
                LOG.log(Level.INFO, "Skipping {0}", file.getAbsoluteFile());
            }
        }
    }
    
    public File raw() {
        return this.tmpFolder;
    }
    
    public boolean createFileCustom(File file) throws IOException {
        this.created.addFirst(file);
        return file.createNewFile();
    }
    
    public boolean createFoldersCustom(File file) {
        this.created.addFirst(file);
        return file.mkdir();
    }
    
    public File get(String name) {
        File result = new File(this.tmpFolder, name);
        this.created.addFirst(result);
        LOG.log(Level.INFO, "Constructed file {0}", result.getAbsolutePath());
        return result;
    }

    public File createFile() throws IOException {
        File result = File.createTempFile("def_", Long.toString(System.nanoTime()), this.tmpFolder);
        this.created.addFirst(result);
        LOG.log(Level.INFO, "Created file {0}", result.getAbsolutePath());
        return result;
    }

    public File createFile(String name) throws IOException {
        File result = new File(this.tmpFolder, name);
        if (!result.createNewFile()) {
            throw new IOException("File " + result.getAbsolutePath() + " can't be created");
        }
        this.created.addFirst(result);
        LOG.log(Level.INFO, "Created file {0}", result.getAbsolutePath());
        return result;
    }

    public File createFolder() throws IOException {
        File result = temp(this.tmpFolder);
        this.created.add(result);
        LOG.log(Level.INFO, "Created folder {0}", result.getAbsolutePath());
        return result;
    }

    public File createFolder(String name) throws IOException {
        File result = new File(this.tmpFolder, name);
        if (!result.mkdir()) {
            throw new IOException("Folder " + result.getAbsolutePath() + " can't be created");
        }
        this.created.addFirst(result);
        LOG.log(Level.INFO, "Created folder {0}", result.getAbsolutePath());
        return result;
    }

}
