/*
 *  Copyright 2013 Kerio Technologies s.r.o. All rights reserved.
 */
package com.kerio.im.core.logging;

import org.junit.Test;
import static org.junit.Assert.*;
import static org.hamcrest.CoreMatchers.*;

/**
 *
 * @author tskoda
 */
public class StanzaAnonymizerTest {

    public StanzaAnonymizerTest() {
    }

    @Test
    public void testAnonymizePositive() {

        StanzaAnonymizer anonymizer = new StanzaAnonymizer();

        //full no html
        assertThat(
                anonymizer.anonymize("Logger.name",
                        "<message att1=\"value1\" att2=\"value2\"><body>anonymize this text</body><other>other</other></message>"),
                is(
                        "<message att1=\"value1\" att2=\"value2\"><body>...</body><other>other</other></message>"));

        //full with html
        assertThat(
                anonymizer.anonymize("Logger.name",
                        "<message att1=\"value1\" att2=\"value2\"><body>anonymize this text</body><html><body>anonymize this text</body></html><other>other</other></message>"),
                is(
                        "<message att1=\"value1\" att2=\"value2\"><body>...</body><html>...</html><other>other</other></message>"));

        //broken beggining and end
        assertThat(
                anonymizer.anonymize("Logger.name",
                        "essage att1=\"value1\" att2=\"value2\"><body>anonymize this text</body><html><body>anonymize this text</body></html><other>other</o"),
                is(
                        "essage att1=\"value1\" att2=\"value2\"><body>...</body><html>...</html><other>other</o"));

        //full no beggining
        assertThat(
                anonymizer.anonymize("Logger.name",
                        "<body>anonymize this text</body><html><body>anonymize this text</body></html><other>other</other></message>"),
                is(
                        "<body>...</body><html>...</html><other>other</other></message>"));

        assertThat(
                anonymizer.anonymize("Logger.name",
                        "body>anonymize this text</body><html><body>anonymize this text</body></html><other>other</other></message>"),
                is(
                        "...</body><html>...</html><other>other</other></message>"));

        assertThat(
                anonymizer.anonymize("Logger.name",
                        "<html><body>anonymize this text</body></html><other>other</other></message>"),
                is(
                        "<html>...</html><other>other</other></message>"));

        assertThat(
                anonymizer.anonymize("Logger.name",
                        "html><body>anonymize this text</body></html><other>other</other></message>"),
                is(
                        "...</html><other>other</other></message>"));

        //full no end
        assertThat(
                anonymizer.anonymize("Logger.name",
                        "<message att1=\"value1\" att2=\"value2\"><body>anonymize this text</body><html><body>anonymize this text</body></html>"),
                is(
                        "<message att1=\"value1\" att2=\"value2\"><body>...</body><html>...</html>"));
        assertThat(
                anonymizer.anonymize("Logger.name",
                        "<message att1=\"value1\" att2=\"value2\"><body>anonymize this text</body><html><body>anonymize this text</body></html"),
                is(
                        "<message att1=\"value1\" att2=\"value2\"><body>...</body><html>..."));

        assertThat(
                anonymizer.anonymize("Logger.name",
                        "<message att1=\"value1\" att2=\"value2\"><body>anonymize this text</body>"),
                is(
                        "<message att1=\"value1\" att2=\"value2\"><body>...</body>"));

        assertThat(
                anonymizer.anonymize("Logger.name",
                        "<message att1=\"value1\" att2=\"value2\"><body>anonymize this text</body"),
                is(
                        "<message att1=\"value1\" att2=\"value2\"><body>..."));

        //some real messages
        assertThat(
                anonymizer.anonymize("Logger.name",
                        "<message from=\"ichat@tskoda-macbook.kerio.local/tskoda-macbook\" id=\"57E3B53B-B31F-4CB5-832F-F18073BB7539\" to=\"pidgin@tskoda-macbook.kerio.local\" xmlns=\"jabber:client\" type=\"chat\"><body>this is content of a message</body>"),
                is(
                        "<message from=\"ichat@tskoda-macbook.kerio.local/tskoda-macbook\" id=\"57E3B53B-B31F-4CB5-832F-F18073BB7539\" to=\"pidgin@tskoda-macbook.kerio.local\" xmlns=\"jabber:client\" type=\"chat\"><body>...</body>"));

        assertThat(
                anonymizer.anonymize("Logger.name",
                        "<html xmlns=\"http://jabber.org/protocol/xhtml-im\"><body style=\"background-color:#e9e9e9;color:#000000;\" xmlns=\"http://www.w3.org/1999/xhtml\"><span style=\"font-family: 'Helvetica';font-size: 12px;\">ale no tak, to neni mozny, ze bych nezachytil tu message tak jak je sakra</span></body></html><x xmlns=\"jabber:x:event\"><composing/></x>"),
                is(
                        "<html>...</html><x xmlns=\"jabber:x:event\"><composing/></x>"));

        assertThat(
                anonymizer.anonymize("Logger.name",
                        "html xmlns=\"http://jabber.org/protocol/xhtml-im\"><body style=\"background-color:#e9e9e9;color:#000000;\" xmlns=\"http://www.w3.org/1999/xhtml\"><span style=\"font-family: 'Helvetica';font-size: 12px;\">ale no tak, to neni mozny, ze bych nezachytil tu message tak jak je sakra</span></body></html><x xmlns=\"jabber:x:event\"><composing/></x>"),
                is(
                        "...</html><x xmlns=\"jabber:x:event\"><composing/></x>"));

    }

    @Test
    public void testAnonymizenegative() {

        StanzaAnonymizer anonymizer = new StanzaAnonymizer();

        assertThat(
                anonymizer.anonymize("Logger.name",
                        "<xml body=\"htmlvalue\"><content html=\"body\""),
                is(
                        "<xml body=\"htmlvalue\"><content html=\"body\""));

        assertThat(
                anonymizer.anonymize("Logger.name",
                        "<xml html=\"body\"><content html=\"body\""),
                is(
                        "<xml html=\"body\"><content html=\"body\""));
    }

}
