/*
 *  Copyright 2013 Kerio Technologies s.r.o. All rights reserved.
 */
package com.kerio.im.core.service;

import com.kerio.im.common.error.Code;
import com.kerio.im.common.error.ServiceException;
import com.kerio.im.external.ExtService;
import org.junit.Test;
import static org.mockito.Mockito.*;

/**
 *
 * @author tskoda
 */
public class ConnectionListenerTest {

    public ConnectionListenerTest() {
    }

    @Test
    public void testDeaf() {

        //test never throw any exception
        ConnectionListener listener = ConnectionListener.deaf();
        listener.connected(null, Boolean.FALSE, null, -1);
        listener.connected("", Boolean.FALSE, "", -1);
        listener.connected("id", Boolean.FALSE, "127.0.0.1", -1);
        listener.connected("id", Boolean.FALSE, "255.255.255.255", -1);

        listener.disconnected(null, null, -1);
        listener.disconnected("", "", -1);
        listener.disconnected("id", "127.0.0.1", -1);
        listener.disconnected("id", "255.255.255.255", -1);
    }

    @Test
    public void testExternalNull() {

        //test must not throw any exception
        ConnectionListener listener = ConnectionListener.external(null);
        listener.connected(null, Boolean.FALSE, null, -1);
        listener.connected("id", Boolean.FALSE, "", -1);
        listener.connected("id", Boolean.FALSE, "127.0.0.1", -1);
        listener.connected("id", Boolean.FALSE, "255.255.255.255", -1);

        listener.disconnected(null, null, -1);
        listener.disconnected("id", "", -1);
        listener.disconnected("id", "127.0.0.1", -1);
        listener.disconnected("id", "255.255.255.255", -1);

    }

    @Test
    public void testExternalExeption() {

        ExtService mockExtService = mock(ExtService.class);
        doThrow(new ServiceException("message", Code.UNKNOWN_ERROR)).when(mockExtService).connect(anyString(), anyString(), anyString(), anyInt(), anyBoolean());
        doThrow(new ServiceException("message", Code.UNKNOWN_ERROR)).when(mockExtService).disconnect(anyString(), anyString());

        //test must not throw any exception
        ConnectionListener listener = ConnectionListener.external(mockExtService);

        listener.connected(null, Boolean.FALSE, null, -1);
        listener.connected("id", Boolean.FALSE, "", -1);
        listener.connected("id", Boolean.FALSE, "127.0.0.1", -1);
        listener.connected("id", Boolean.FALSE, "255.255.255.255", -1);

        listener.disconnected(null, null, -1);
        listener.disconnected("id", "", -1);
        listener.disconnected("id", "127.0.0.1", -1);
        listener.disconnected("id", "255.255.255.255", -1);
    }

    @Test
    public void testExternalNoIP() {

        ExtService mockExtService = mock(ExtService.class);
        ConnectionListener listener = ConnectionListener.external(mockExtService);

        String serviceId = "service-id";
        Boolean secure = Boolean.FALSE;


        listener.connected(serviceId, secure, null, -1);
        verify(mockExtService, times(0)).connect(anyString(), anyString(), anyString(), anyInt(), anyBoolean());

        listener.disconnected(serviceId, null, -1);
        verify(mockExtService, times(0)).disconnect(anyString(), anyString());
    }

    @Test
    public void testExternalNoPort() {

        ExtService mockExtService = mock(ExtService.class);
        ConnectionListener listener = ConnectionListener.external(mockExtService);

        String serviceId = "service-id";
        Boolean secure = Boolean.FALSE;
        String ip = "127.0.0.1";


        listener.connected(serviceId, secure, ip, -1);
        verify(mockExtService, times(0)).connect(anyString(), anyString(), anyString(), anyInt(), anyBoolean());

        listener.disconnected(serviceId, ip, -1);
        verify(mockExtService, times(0)).disconnect(anyString(), anyString());
    }

    @Test
    public void testExternal() {

        ExtService mockExtService = mock(ExtService.class);
        ConnectionListener listener = ConnectionListener.external(mockExtService);

        String serviceId = "service-id";
        Boolean secure = Boolean.TRUE;
        String ip = "127.0.0.1";
        int port = 68756;

        listener.connected(serviceId, secure, ip, port);
        verify(mockExtService, times(1)).connect(serviceId, ConnectionListener.asConnectinId(ip, port), ip, port, secure);

        listener.disconnected(serviceId, ip, port);
        verify(mockExtService, times(1)).disconnect(serviceId, ConnectionListener.asConnectinId(ip, port));

    }
}
