/*
 * Copyright 2014 Kerio Technologies s.r.o. All rights reserved
 */
package com.kerio.im.core.service;

import org.junit.After;
import org.junit.AfterClass;
import org.junit.Before;
import org.junit.BeforeClass;
import org.junit.Test;
import static org.junit.Assert.*;

/**
 *
 * @author tskoda
 */
public class SecurityManagerTest {
    
    /**
     * Test of filterCiphers method, of class SecurityManager.
     */
    @Test
    public void testFilterCiphers() {
    }

    /**
     * Test of filterProtocols method, of class SecurityManager.
     */
    @Test
    public void testFilterProtocols() {
        SecurityManager sm = new SecurityManager();
        String[] expected = new String[]{"SSLv2Hello", "TLSv1", "TLSv1.1", "TLSv1.2"};
        assertArrayEquals(expected, sm.filterProtocols(new String[]{"SSLv3", "SSLv2Hello", "SSLv3", "TLSv1", "SSLv3", "TLSv1.1", "SSLv3", "TLSv1.2", "SSLv3"}));
        //the result is cached, the list of supported protocols is static for JVM lifetime
        assertArrayEquals(expected, sm.filterProtocols(new String[]{"SSLv3"}));
        assertArrayEquals(expected, sm.filterProtocols(null));
    }
    
    /**
     * Test of filterProtocols method, of class SecurityManager.
     */
    @Test
    public void testFilterProtocolsNull() {
        SecurityManager sm = new SecurityManager();
        String[] expected = null;
        assertArrayEquals(expected, sm.filterProtocols(null));
        //the result is cached, the list of supported protocols is static for JVM lifetime
        expected = new String[0];
        assertArrayEquals(expected, sm.filterProtocols(new String[]{"SSLv3"}));
        assertArrayEquals(expected, sm.filterProtocols(new String[]{"SSLv3", "SSLv2Hello", "SSLv3", "TLSv1", "SSLv3", "TLSv1.1", "SSLv3", "TLSv1.2", "SSLv3"}));
    }
    
    /**
     * Test of filterProtocols method, of class SecurityManager.
     */
    @Test
    public void testFilterProtocolsEmpty() {
        SecurityManager sm = new SecurityManager();
        String[] expected = new String[0];
        assertArrayEquals(expected, sm.filterProtocols(new String[0]));
        //the result is cached, the list of supported protocols is static for JVM lifetime
        assertArrayEquals(expected, sm.filterProtocols(new String[]{"SSLv3"}));
        assertArrayEquals(expected, sm.filterProtocols(new String[]{"SSLv3", "SSLv2Hello", "SSLv3", "TLSv1", "SSLv3", "TLSv1.1", "SSLv3", "TLSv1.2", "SSLv3"}));
    }
    
    /**
     * Test of filterProtocols method, of class SecurityManager.
     */
    @Test
    public void testFilterProtocolsSSLv3Only() {
        SecurityManager sm = new SecurityManager();
        String[] expected = new String[0];
        assertArrayEquals(expected, sm.filterProtocols(new String[]{"SSLv3"}));
        //the result is cached, the list of supported protocols is static for JVM lifetime
        assertArrayEquals(expected, sm.filterProtocols(new String[]{"SSLv3", "SSLv2Hello", "SSLv3", "TLSv1", "SSLv3", "TLSv1.1", "SSLv3", "TLSv1.2", "SSLv3"}));
    }
    
}
