/*
 * Copyright 2013 Kerio Technologies s.r.o. All rights reserved.
 */
package com.kerio.im.connect;

import com.kerio.lib.json.api.client.KJsonRpcSession;
import java.io.ByteArrayOutputStream;
import java.io.IOException;
import java.io.InputStream;
import java.io.OutputStream;
import java.net.HttpURLConnection;
import java.net.MalformedURLException;
import java.net.ProtocolException;
import java.net.URL;
import java.util.logging.Level;
import java.util.logging.Logger;
import javax.imageio.IIOException;

/**
 *
 * @author tskoda
 */
public class FileApiClient {

    private static final Logger LOG = Logger.getLogger(FileApiClient.class.getName());
    private ApiClient.KJsonRpcSessionPrivate session;

    public FileApiClient(ApiClient.KJsonRpcSessionPrivate session) {
        this.session = session;
    }

    private HttpURLConnection connect(String fileURL) throws IOException {
        URL url = new URL(fileURL);
        HttpURLConnection con = (HttpURLConnection) url.openConnection();

        con.setAllowUserInteraction(false);
        con.setDefaultUseCaches(false);
        con.setDoInput(true);
        con.setDoOutput(false);
        con.setUseCaches(false);
        con.setInstanceFollowRedirects(true);
        con.setRequestMethod("GET");
        con.setRequestProperty("Cookie", this.session.getCookie());//this.adminApiClient.getSession().getCookie());
        con.setRequestProperty("X-Token", this.session.getToken());//this.adminApiClient.getSession().getToken());
        con.connect();
        return con;
    }

    public String fileContent(String fileURL, OutputStream out) throws MalformedURLException, IOException {
        LOG.log(Level.FINE, "Requesting file content {0}", fileURL);
        String contentType = null;
        try {

            HttpURLConnection con = connect(fileURL);

            int responseCode = con.getResponseCode();
            if (responseCode == 403) {
                this.session.reconnect();
                con = connect(fileURL);
                responseCode = con.getResponseCode();
            }

            if (responseCode == 200) {
                byte[] buffer = new byte[512];
                int count = 0;

                InputStream fileInputStream = con.getInputStream();

                try {
                    while ((count = fileInputStream.read(buffer)) > 0) {
                        out.write(buffer, 0, count);
                    }

                    if (con.getContentType() != null) {
                        if (con.getContentType().contains(";")) {
                            contentType = con.getContentType().substring(0, con.getContentType().indexOf(";"));
                        } else {
                            contentType = con.getContentType();
                        }
                    }

                } finally {
                    try {
                        fileInputStream.close();
                    } catch (IOException e) {
                        LOG.warning("Can not close url input stream");
                    }
                }

            } else {
                throw new IOException("Failed to obtain file from " + fileURL + ", response code " + responseCode);
            }
        } finally {
            try {
                out.close();
            } catch (IOException e) {
                LOG.warning("Can not close url output stream");
            }
        }

        LOG.log(Level.FINE, "Requesting file content successful");

        return contentType;
    }

    public byte[] asByteArray(String fileURL) throws MalformedURLException, IOException {
        ByteArrayOutputStream buffer = new ByteArrayOutputStream();
        this.fileContent(fileURL, buffer);
        return buffer.toByteArray();
    }

    public String asString(String fileURL) throws MalformedURLException, IOException {
        ByteArrayOutputStream buffer = new ByteArrayOutputStream();
        this.fileContent(fileURL, buffer);
        return buffer.toString();
    }
}
