package com.kerio.im.im;

import com.kerio.lib.json.api.client.KJsonRpcSession;
import com.kerio.lib.json.api.client.KStreamUtil;
import java.io.IOException;
import java.io.InputStream;
import java.io.OutputStream;
import java.lang.reflect.Method;
import java.net.HttpURLConnection;
import java.net.URL;
import org.jabsorb.client.ClientError;
import org.json.JSONException;
import org.json.JSONObject;

class ImJsonRpcSession extends KJsonRpcSession {
    private final String user;
    private final String sessionid;
    private String registerid;
    private static final int CONNECTION_TIMEOUT = 180 * 1000;
    private static final int READ_TIMEOUT = 180 * 1000;
    
    public ImJsonRpcSession(String hostname, String apiPath, String user, String sessionid, String registerid) {
        this.user = user;
        this.sessionid = sessionid;
        this.registerid = registerid;
        setHostname(hostname);
        setApiPath(apiPath);
    }

    public void setRegisterid(String registerid) {
        this.registerid = registerid;
    }

    public String getSessionid() {
        return sessionid;
    }

    @Override
    public JSONObject sendAndReceive(JSONObject message, Method method) {
        HttpURLConnection con = null;
        InputStream in = null;
        OutputStream out = null;

        try {
            URL url = new URL(getUrl());

            con = (HttpURLConnection) url.openConnection();
            con.setAllowUserInteraction(false);
            con.setDefaultUseCaches(false);
            con.setDoInput(true);
            con.setDoOutput(true);
            con.setUseCaches(false);
            con.setInstanceFollowRedirects(true);
            con.setRequestMethod("POST");
            con.setRequestProperty("Content-Type", "application/json-rpc; charset=UTF-8");
            con.setRequestProperty("Accept", "application/json-rpc");
            con.setRequestProperty("X-Requested-With", "XMLHttpRequest");

            con.setRequestProperty("X-User-ID", user);
            con.setRequestProperty("X-Session-ID", sessionid);
            con.setRequestProperty("X-Server-ID", registerid);
            con.setReadTimeout(READ_TIMEOUT);
            con.setConnectTimeout(CONNECTION_TIMEOUT);
            con.connect();

            out = con.getOutputStream();
            out.write(message.toString().getBytes("UTF-8"));
            out.flush();

            JSONObject response;
            int responseCode = con.getResponseCode();
            if (responseCode == HttpURLConnection.HTTP_OK) {
                in = con.getInputStream();
                response = new JSONObject(KStreamUtil.readToString(in));
            }
            else {
                String msg;
                try {
                    in = con.getErrorStream();
                    msg = KStreamUtil.readToString(in);
                    response = new JSONObject(msg);
                } catch (Throwable e) {
                    msg = e.getMessage();
                    response = new JSONObject("{}");
                }
                throw new JsonErrorException(responseCode, response, msg);
            }

            return response;
        } catch (JsonErrorException e) {
            throw e;
        } catch (RuntimeException e) {
            throw e;
        } catch (IOException | JSONException e) {
            throw new ClientError(e);
        } finally {
            if (con != null) {
                con.disconnect();
            }

            KStreamUtil.closeNoThrow(in);
            KStreamUtil.closeNoThrow(out);
        }
    }
}
