/*
 * Copyright 2013 Kerio Technologies s.r.o. All rights reserved.
 */
package com.kerio.im.tigase;

import com.kerio.im.bean.Port;
import com.kerio.im.core.api.service.AuthService;
import com.kerio.im.tigase.bridge.xmpp.IAuthServiceProvider;
import com.kerio.im.tigase.bridge.xmpp.IMBridge;
import com.kerio.im.tigase.bridge.xmpp.IXmppConnectionManager;
import tigase.net.SocketType;
import tigase.server.ConnectionManager;
import tigase.server.xmppclient.ClientConnectionManager;
import tigase.xmpp.XMPPIOService;

/**
 *
 * @author tskoda
 */
public class ClientResolver {

	public static final String CLIENT_TYPE = "client.type";

	public enum ClientType {

		ichat,
		unknown;

		public static ClientType resolve(String id) {
			ClientType result;
			if (id.toLowerCase().startsWith("ichat")) {
				result = ClientType.ichat;
			} else {
				result = ClientType.unknown;
			}
			return result;
		}
	}
	private final ComponentLookup lookup;

	public ClientResolver(ComponentLookup lookup) {
		this.lookup = lookup;
	}

	public AuthService.Connection resolve(String identifier) {
        XMPPIOService<Object> ioService = lookup.lookup(ClientConnectionManager.class).getXMPPIOService(identifier);
        if (ioService != null) {
             return new AuthService.Connection(
                getLocalPort(ioService),
                isSecure(ioService),
                ioService.getRemoteAddress(), //remote address contains address from which client is connecting, e.g. localhost instead of 127.0.0.1
                ioService.getRemotePort(),
                tlsRequired(ioService));
        }

        IAuthServiceProvider fakeConnectionsManager = lookup.lookup(IMBridge.class).getConnectionManager();
        if (fakeConnectionsManager != null) {
            return fakeConnectionsManager.getAuthServiceForConnection(identifier);
        }
        return null;
	}      

	public boolean isSecure(String identifier) {
		XMPPIOService<Object> ioService = lookup.lookup(ClientConnectionManager.class).getXMPPIOService(identifier);
		return isSecure(ioService);
	}

	private boolean isSecure(XMPPIOService<Object> ioService) {
		SocketType socketType = SocketType.valueOf(ioService.getSessionData().get(ConnectionManager.PORT_SOCKET_PROP_KEY).toString());
		return socketType == SocketType.ssl;
	}

	private int getLocalPort(XMPPIOService<Object> ioService) {
		Object portObject = ioService.getSessionData().get(ConnectionManager.PORT_KEY);
		int result = 0;
		if (portObject != null) {
			result = (Integer) portObject;
		}
		return result;
	}

	private boolean tlsRequired(XMPPIOService<Object> ioService) {
		Object securityObject = ioService.getSessionData().get(PortServiceImpl.SECURITY);
		return securityObject != null ? ((Port.Security) securityObject).requireTls(ioService.getSocketChannel().socket().getInetAddress()) : true;
		//return securityObject != null? ((Port.Security)securityObject).requireTLS(): true;
	}

        public ComponentLookup getLookup() {
            return lookup;
        }               
}
