/*
 * Copyright 2014 Kerio Technologies s.r.o. All rights reserved
 */
package com.kerio.im.tigase.bridge.IM;

import com.kerio.im.external.IMContactID;
import com.kerio.im.external.ImService;
import com.kerio.im.tigase.bridge.IMSubscribctionsMamager;
import com.kerio.im.tigase.bridge.xmpp.IEventsConsumerEx;
import com.kerio.im.tigase.bridge.xmpp.IMBridge;
import java.util.List;
import java.util.TreeMap;
import java.util.logging.Level;
import java.util.logging.Logger;
import java.util.stream.Collectors;
import tigase.xmpp.BareJID;

/**
 *
 * @author vbystricky
 */
public class IMConnectionManager {

    private static final Logger LOGGER = Logger.getLogger(IMConnectionManager.class.getName());

    private final ImService imService;
    private final IEventsConsumerEx eventConsumer;
    private final IMSubscribctionsMamager subscribctionsMamager;
    private final TreeMap<BareJID, IMConnection> connections;

    public IMConnectionManager(ImService imService, IEventsConsumerEx eventConsumer, IMSubscribctionsMamager subscribctionsMamager) {
        this.connections = new TreeMap<>();
        this.imService = imService;
        this.eventConsumer = eventConsumer;
        this.subscribctionsMamager = subscribctionsMamager;
    }

    public IMConnection getConnection(BareJID userId) {
        return connections.get(userId);
    }

    public IMConnection createConnection(BareJID userId) {
        IMConnection result = getConnection(userId);
        if (result == null) {
            result = new IMConnection(userId, imService.createUserImClient(new IMContactID(userId.toString())), eventConsumer, subscribctionsMamager);
            connections.put(userId, result);
        }
        return result;
    }

    public void removeConnection(BareJID userId) {
        connections.remove(userId);
    }

    public List<UserPresence> getUserPresences() {
        return connections.values().stream().map(connection -> new UserPresence(connection.getUserId(), connection.getClientsCopy())).collect(Collectors.toList());
    }

    public void destroyAllConnections(boolean restart) {
        LOGGER.log(Level.FINE, IMBridge.LOG_PREFIX + "Closing all IM connections");
        connections.values().stream().forEach((connection) -> {
            connection.close(restart);
        });
        connections.clear();
    }

    public IMServerConnection getServerConnection() {
        return new IMServerConnection(imService);
    }
}
