/*
 * Copyright 2014 Kerio Technologies s.r.o. All rights reserved
 */
package com.kerio.im.tigase.bridge.conversation;

import com.kerio.im.external.IMContactID;
import com.kerio.im.tigase.bridge.xmpp.IMBridge;
import java.util.logging.Level;
import java.util.logging.Logger;
import tigase.xmpp.JID;

/**
 *
 * @author jzacek
 */
public class Conversation {

    private static final Logger LOGGER = Logger.getLogger(Conversation.class.getName());
    private final ConversationJID user1;
    private final ConversationJID user2;
    private final Integer conversationId;

    public Conversation(ConversationJID user1, ConversationJID user2, Integer conversationId) {
        if (user1.compareTo(user2) > 0) {
            this.user2 = user1;
            this.user1 = user2;
        } else {
            this.user1 = user1;
            this.user2 = user2;
        }
        this.conversationId = conversationId;
    }

    public ConversationJID getUser1() {
        return user1;
    }

    public ConversationJID getUser2() {
        return user2;
    }

    public ConversationJID getOther(ConversationJID c) {
        return c.equalsWithoutResource(user1) ? user2 : user1;
    }

    public ConversationJID getOther(IMContactID c) {
        return user1.equalsWithoutResource(new ConversationJID(c)) ? user2 : user1;
    }

    public Integer getConversationId() {
        return conversationId;
    }

    @Override
    public String toString() {
        return "ID: " + conversationId + " user1: " + user1 + " user2: " + user2;
    }

    public synchronized void updateSenderResource(JID sender) {
        ConversationJID tmp = null;
        if (user1.equalsWithoutResource(sender)) {
            tmp = user1;
        } else if (user2.equalsWithoutResource(sender)) {
            tmp = user2;
        }
        if (tmp != null) {
            LOGGER.log(Level.FINE, IMBridge.LOG_PREFIX + "Updating resource to {0} for user {1} in conversation {2}.", new Object[]{sender.getResource(), tmp, this.toString()});
            tmp.setResource(sender.getResource());
        } else {
            LOGGER.log(Level.INFO, IMBridge.LOG_PREFIX + "Can't update resource for sender {0} in conversation {1}.", new Object[]{sender, this.toString()});
        }
    }
}
