/*
 * Copyright 2014 Kerio Technologies s.r.o. All rights reserved
 */
package com.kerio.im.tigase.bridge.xmpp;

import com.kerio.im.tigase.bridge.communicator.IMtoXmppChannel;
import tigase.util.TigaseStringprepException;
import tigase.xmpp.JID;

/**
 *
 * @author vbystricky
 */
public class OfflineDeliveryXmppConnection implements IXmppConnection {

    private final JID userId;
    private final String SESSION_MANAGER_ID;
    private final IMtoXmppChannel communicator;

    public OfflineDeliveryXmppConnection(JID userId, String SESSION_MANAGER_ID, IMtoXmppChannel communicator) {
        this.userId = userId;
        this.SESSION_MANAGER_ID = SESSION_MANAGER_ID;
        this.communicator = communicator;
    }

    @Override
    public boolean open() throws TigaseStringprepException {
        return true;
    }

    @Override
    public boolean setPresence(XmppPresence presence, String text) throws TigaseStringprepException {
        return true;
    }

    private String getPacketID() {
        return XmppIdGenerator.generatePacketID();
    }

    @Override
    public boolean sendMessage(JID receiver, String text) throws TigaseStringprepException {
        this.communicator.sendMessage(PacketFactory.getMessagePacket(getUserJID(), receiver, text, getPacketID(), null, JID.jidInstance(SESSION_MANAGER_ID)));
        return true;
    }

    @Override
    public void close() {
    }

    @Override
    public String getConnectionID() {
        return "";
    }

    @Override
    public JID getUserJID() {
        return userId;
    }

    @Override
    public String getFullUserName() {
        return userId.toString();
    }

    @Override
    public String getUser() {
        return userId.getLocalpart();
    }

    @Override
    public String getDomain() {
        return userId.getDomain();
    }

    @Override
    public String getResource() {
        return null;
    }
}
