/*
 * Copyright 2014 Kerio Technologies s.r.o. All rights reserved
 */
package com.kerio.im.tigase.bridge.xmpp;

import com.kerio.im.external.IMPresence;
import java.util.Arrays;

/**
 *
 * @author jzacek
 */
public enum XmppPresence {
    AVAILABLE("available"), AWAY("away"), DO_NOT_DISTURB("dnd"), EXTENDED_AWAY("xa"), OFFLINE("offline"), CHAT("chat");

    private final String text;

    XmppPresence(String text) {
        this.text = text;
    }

    public static XmppPresence get(String p) {
        Object[] res = Arrays.asList(XmppPresence.class.getEnumConstants()).stream()
                .filter((t) -> t.toString().equalsIgnoreCase(p)).toArray();
        return res.length == 1 ? (XmppPresence) res[0] : null;
    }

    public static IMPresence.Status getIMPresence(XmppPresence presence) throws IllegalArgumentException {
        switch (presence) {
            case AVAILABLE:
            case CHAT:
                return IMPresence.Status.available;
            case EXTENDED_AWAY:
            case AWAY:
                return IMPresence.Status.away;
            case DO_NOT_DISTURB:
                return IMPresence.Status.dnd;
            case OFFLINE:
                return IMPresence.Status.offline;
            default:
                throw new IllegalArgumentException("Unsuported xmpp presence status " + presence);
        }
    }

    public static XmppPresence getXmppPresence(IMPresence.Status status) throws IllegalArgumentException {
        switch (status) {
            case available:
                return XmppPresence.AVAILABLE;
            case offline:
                return XmppPresence.OFFLINE;
            case dnd:
                return XmppPresence.DO_NOT_DISTURB;
            case away:
                return XmppPresence.AWAY;
            case invisible:
                return XmppPresence.OFFLINE;
            default:
                throw new IllegalArgumentException("Unsuported im presence status " + status);
        }
    }

    @Override
    public String toString() {
        return this.text;
    }
}
