/*
 * Copyright 2013 Kerio Technologies s.r.o. All rights reserved.
 */
package com.kerio.im.tigase.configuration;

import com.kerio.im.tigase.ConnectComponent;
import com.kerio.im.tigase.DomainServiceImpl;
import com.kerio.im.tigase.auth.ConnectAuthenticator;
import com.kerio.im.tigase.bridge.xmpp.IMBridge;
import com.kerio.im.xmpp.api.XmppConstants;
import java.io.File;
import tigase.conf.Configurable;

/**
 *
 * @author tskoda
 */
public enum TigaseKey {

    CONFIG_TYPE             ("config-type", "--gen-config-def"),
    VIRTUAL_HOST            ("--virt-hosts", "%1$s"),
    CLUSTER_MODE            ("--cluster-mode", "false"),
    USER_DB                 ("--user-db", "derby"),
    USER_DB_URI             ("--user-db-uri", "jdbc:derby:%1$s" + File.separator + DomainServiceImpl.DB_ROOT + File.separator + "%2$s"), //data root, domain name
    SCRIPTS_FOLDER          (Configurable.GEN_SCRIPT_DIR, "%1$s" + File.separator + "scripts"),
    SM_THREADS_POOL         ("--sm-threads-pool", "custom:10"),
    SM_PLUGINS              ("--sm-plugins", "-starttls,+connect-starttls,-vcard-temp,+connect-vcard,-jabber:iq:auth,-urn:ietf:params:xml:ns:xmpp-sasl,+connect-sasl,+urn:ietf:params:xml:ns:xmpp-bind,+urn:ietf:params:xml:ns:xmpp-session,-jabber:iq:register,+jabber:iq:roster,+presence,+jabber:iq:privacy,+jabber:iq:version,-http://jabber.org/protocol/stats,-msgoffline,+amp,-http://jabber.org/protocol/commands,+jabber:iq:private,+urn:xmpp:ping,+connect-ping,-basic-filter,-domain-filter,+connect-domain-filter,+connect-vcard-fix,+connect-client-resolver,-pep,-zlib,+disconnect-listener,+im-bridge-synchronizator"),
    DERBY_SYSTEM_HOME       ("--derby.system.home", "%1$s"),
    SSL_DEF_CERT_DOMAIN     ("--ssl-def-cert-domain", "%1$s"),
    AUTH_DB                 ("--auth-db", ConnectAuthenticator.class.getName()),
    ADMINS                  ("--admins", "%1$s"),
    ENGINE_COMP             ("--comp-name-1", ConnectComponent.COMPONENT_NAME),
    ENGINE_COMP_CLASS       ("--comp-class-1", ConnectComponent.class.getName()),
    MUC_COMP                ("--comp-name-2", XmppConstants.GROUP_CHAT_NAME),
    MUC_COMP_CLASS          ("--comp-class-2", "tigase.muc.MUCComponent"),
    IM_BRIDGE_COMP          ("--comp-name-3", IMBridge.IM_BRIDGE_NAME),    
    IM_BRIDGE_COMP_CLASS    ("--comp-class-3", IMBridge.class.getName()),    
    MUC_UNLOCKED            ("conference/muc-lock-new-room[B]", "false"),
    BOSH_COMPONENT_STATE    ("message-router/components/msg-receivers/bosh.active[B]", "false"),
    MONITOR_COMPONENT_STATE ("message-router/components/msg-receivers/monitor.active[B]", "false"),
    S2S_COMPONENT_STATE     ("message-router/components/msg-receivers/s2s.active[B]", "%1$s"),
    UPDATE_CHECKING         ("message-router/updates-checking[B]", "false");
    
    
    public final String key;
    public final String value;
    

    private TigaseKey(String key, String value) {
        this.key = key;
        this.value = value;
    }
    
}
