/*
 * Copyright 2013 Kerio Technologies s.r.o. All rights reserved.
 */
package com.kerio.im.tigase.configuration.task;

import com.kerio.im.common.Task;
import com.kerio.im.common.error.Code;
import com.kerio.im.core.api.GlobalKey;
import com.kerio.im.core.api.service.ConfigurationService;
import com.kerio.im.tigase.configuration.PropertyProcessor;
import com.kerio.im.xmpp.api.XmppException;
import java.io.File;
import java.util.logging.Level;
import java.util.logging.Logger;

/**
 *
 * @author tskoda
 */
public class AddCertificatePaths implements Task<PropertyProcessor> {

    private static final Logger LOG = Logger.getLogger(AddCertificatePaths.class.getName());
    
    private final ConfigurationService configuration;
    
    public AddCertificatePaths(ConfigurationService configuration) {
        this.configuration = configuration;
    }

    @Override
    public void run(PropertyProcessor processor) {
        LOG.log(Level.CONFIG, "Adding certificate paths");
      
        File dataDir = configuration.get(GlobalKey.DATA_DIR);
        
        File certificates = mkdir(new File(dataDir, "certs"));
        String rsaKeystore = new File(certificates, "rsa-keystore").getAbsolutePath();
        String truststore = new File(certificates, "truststore").getAbsolutePath();

        processor.put("--ssl-certs-location", certificates.getAbsolutePath());
        processor.put("s2s/connections/tls/keys-store", rsaKeystore);
        processor.put("s2s/connections/tls/trusts-store", truststore);
        processor.put("c2s/connections/tls/keys-store", rsaKeystore);
        processor.put("c2s/connections/tls/trusts-store", truststore);
        processor.put("bosh/connections/tls/keys-store", rsaKeystore);
        processor.put("bosh/connections/tls/trusts-store", truststore);
        
    }
    
    protected File mkdir(File folder) {
        if (!folder.exists() && !folder.mkdir()) {
            throw new XmppException("Failed to create {0}", Code.RESOURCE_ERROR, folder.getAbsolutePath());
        }
        
        return folder;
    }
}
