/*
 * Copyright 2013 Kerio Technologies s.r.o. All rights reserved.
 */
package com.kerio.im.tigase.configuration.task;

import com.kerio.im.common.Task;
import com.kerio.im.common.error.Code;
import com.kerio.im.core.api.GlobalKey;
import com.kerio.im.core.api.service.ConfigurationService;
import com.kerio.im.tigase.configuration.PropertyProcessor;
import com.kerio.im.xmpp.api.XmppException;
import java.io.File;
import java.io.FileOutputStream;
import java.io.IOException;
import java.util.logging.Level;
import java.util.logging.Logger;

/**
 *
 * @author tskoda
 */
public class AddCrossDomainPolicy implements Task<PropertyProcessor> {
    
    private static final Logger LOG = Logger.getLogger(AddCrossDomainPolicy.class.getName());
    private static final String DEFAULT_CONTENT = ""
            + "<!DOCTYPE cross-domain-policy SYSTEM \"http://www.macromedia.com/xml/dtds/cross-domain-policy.dtd\">\n"
            + "<cross-domain-policy>\n"
            + "<allow-access-from domain=\"*\" to-ports=\"80,8080,5222,5223\"/>\n"
            + "<site-control permitted-cross-domain-policies=\"all\"/>\n"
            + "</cross-domain-policy>\n";
    
    private final ConfigurationService configuration;
    
    public AddCrossDomainPolicy(ConfigurationService configuration) {
        this.configuration = configuration;
    }

    @Override
    public void run(PropertyProcessor processor) {
        LOG.log(Level.CONFIG, "Adding cross domain policy file");
        
        //TODO put lookup outside, share mkdir function
        File dataDir = configuration.get(GlobalKey.DATA_DIR);

        File etc = mkdir(new File(dataDir, "etc"));
        File extraHeaders = mkfile(new File(etc, "cross-domain-policy.xml"));
        processor.put("--cross-domain-policy-file", extraHeaders.getAbsolutePath());
    }

    protected File mkdir(File folder) {
        if (!folder.exists() && !folder.mkdir()) {
            throw new XmppException("Failed to create {0}", Code.RESOURCE_ERROR, folder.getAbsolutePath());
        }

        return folder;
    }
    
    protected File mkfile(File bosh) {
        if (!bosh.exists()) {
            FileOutputStream fos = null;
            try {
                LOG.info("Creating cross domain policy file with default content");
                fos = new FileOutputStream(bosh);
                fos.write(DEFAULT_CONTENT.getBytes());
                fos.flush();
            } catch (IOException e) {
                
            } finally {
                if (fos != null) {
                    try {
                        fos.close();
                    } catch (IOException e) {
                        LOG.log(Level.WARNING, "Can not close {0} file output", bosh.getAbsolutePath());
                    }
                }
            }
        }
        
        return bosh;
    }
    
}
