/*
 * Copyright 2013 Kerio Technologies s.r.o. All rights reserved.
 */
package com.kerio.im.tigase.configuration.task;

import com.kerio.im.common.Task;
import com.kerio.im.common.error.Code;
import com.kerio.im.core.api.GlobalKey;
import com.kerio.im.core.api.service.ConfigurationService;
import com.kerio.im.tigase.DomainServiceImpl;
import com.kerio.im.xmpp.api.XmppException;
import java.io.File;
import java.util.logging.Level;
import java.util.logging.Logger;

/**
 *
 * @author tskoda
 */
public class AddDatabaseRoot implements Task<ConfigurationService> {
    
    private static final Logger LOG = Logger.getLogger(AddDatabaseRoot.class.getName());

    @Override
    public void run(ConfigurationService configuration) {
        File dbRoot = new File(configuration.get(GlobalKey.DATA_DIR), DomainServiceImpl.DB_ROOT);
        LOG.log(Level.CONFIG, "Checking database root folder {0}", dbRoot.getAbsolutePath());
        //parent folder must exist
        if (! dbRoot.exists() && ! dbRoot.mkdir()) {
            throw new XmppException("Checking database root folder failed, can not create folder {0}", Code.RESOURCE_ERROR, dbRoot.getAbsoluteFile());
        
        } 
        
        if (! dbRoot.isDirectory() || ! dbRoot.canRead() || ! dbRoot.canWrite()) {
            throw new XmppException("Checking database root folder failed, folder {0} is not accessible", Code.RESOURCE_ERROR, dbRoot.getAbsoluteFile());
        }
    }
    
    
    
}
