/*
 * Copyright 2013 Kerio Technologies s.r.o. All rights reserved.
 */
package com.kerio.im.tigase.configuration.task;

import com.kerio.im.common.Lookup;
import com.kerio.im.common.Task;
import com.kerio.im.core.api.GlobalKey;
import com.kerio.im.core.api.service.ConfigurationService;
import com.kerio.im.tigase.configuration.PropertyProcessor;
import com.kerio.im.tigase.configuration.TigaseKey;
import java.util.EnumSet;
import java.util.Set;
import java.util.logging.Level;
import java.util.logging.Logger;

/**
 *
 * @author tskoda
 */
public class ApplyTigaseProperties implements Task<PropertyProcessor> {

    private static final Logger LOG = Logger.getLogger(ApplyTigaseProperties.class.getName());
    private Set<TigaseKey> unprocessed;
    private final ConfigurationService configuration;

    public ApplyTigaseProperties(ConfigurationService configuration) {
        this.configuration = configuration;
    }

    @Override
    public void run(PropertyProcessor processor) {

        LOG.log(Level.CONFIG, "Applying default Tigase configuration values: {0}", TigaseKey.values());

        unprocessed = EnumSet.allOf(TigaseKey.class);

        String domain = configuration.get(GlobalKey.SERVICE_REMOTE_DOMAIN);
        String serviceDir = configuration.get(GlobalKey.SERVICE_DIR).getAbsolutePath();
        String dataDir = configuration.get(GlobalKey.DATA_DIR).getAbsolutePath();
        String admins = configuration.get(GlobalKey.SERVICE_REMOTE_LOGIN) + "@" + domain;
        Boolean s2s = Boolean.TRUE; //by default its enabled but without open ports

        apply(processor, TigaseKey.CONFIG_TYPE);
        apply(processor, TigaseKey.VIRTUAL_HOST, domain);
        apply(processor, TigaseKey.CLUSTER_MODE);
        apply(processor, TigaseKey.USER_DB);
        apply(processor, TigaseKey.SCRIPTS_FOLDER, dataDir);
        apply(processor, TigaseKey.USER_DB_URI, dataDir, domain);//             ("--user-db-uri ", "jdbc:derby:%1$s/tigasedb"),
        apply(processor, TigaseKey.SM_THREADS_POOL);
        apply(processor, TigaseKey.SM_PLUGINS);//
        apply(processor, TigaseKey.DERBY_SYSTEM_HOME, serviceDir); //       ("--derby.system.home", "%1$s"),
        apply(processor, TigaseKey.SSL_DEF_CERT_DOMAIN, domain); //     ("--ssl-def-cert-domain", "%1$s"),
        apply(processor, TigaseKey.AUTH_DB);
        apply(processor, TigaseKey.ADMINS, admins);  //                ("--admins", null),
        apply(processor, TigaseKey.ENGINE_COMP);
        apply(processor, TigaseKey.ENGINE_COMP_CLASS);
        apply(processor, TigaseKey.MUC_COMP);
        apply(processor, TigaseKey.MUC_COMP_CLASS);
        apply(processor, TigaseKey.MUC_UNLOCKED);
        apply(processor, TigaseKey.IM_BRIDGE_COMP);
        apply(processor, TigaseKey.IM_BRIDGE_COMP_CLASS);        
        apply(processor, TigaseKey.BOSH_COMPONENT_STATE);
        apply(processor, TigaseKey.MONITOR_COMPONENT_STATE);
        apply(processor, TigaseKey.S2S_COMPONENT_STATE, s2s);//    ("message-router/components/msg-receivers/s2s.active[B]", "%1$s"),
        apply(processor, TigaseKey.UPDATE_CHECKING);        

        for (TigaseKey key : unprocessed) {
            LOG.log(Level.WARNING, "Skipped tigase configuration key {0}", key.key);
        }

    }

    protected void apply(PropertyProcessor processor, TigaseKey key, Object... valueParameters) {

        String value = format(key.value, valueParameters);

        processor.put(key.key, value);
        this.unprocessed.remove(key);
    }

    protected void apply(PropertyProcessor processor, TigaseKey key, Object[] keyParameters, Object[] valueParameters) {

        String keyValue = format(key.key, keyParameters);
        String value = format(key.value, valueParameters);

        processor.put(keyValue, value);
        this.unprocessed.remove(key);
    }

    private String format(String value, Object[] parameters) {
        return (parameters == null || parameters.length == 0) ? value : String.format(value, parameters);
    }
}
