/*
 *  Copyright 2013 Kerio Technologies s.r.o. All rights reserved.
 */
package com.kerio.im.tigase.configuration.task;

import com.kerio.im.common.Task;
import com.kerio.im.common.error.Code;
import com.kerio.im.tigase.configuration.PluginConfigurator;
import com.kerio.im.tigase.roster.DynamicDomainRoster;
import com.kerio.im.tigase.roster.DynamicDomainRosterHandle;
import com.kerio.im.tigase.roster.IDynamicDomainRoster;
import com.kerio.im.tigase.roster.IPersonalRosterListener;
import com.kerio.im.xmpp.api.XmppException;
import tigase.db.TigaseDBException;
import tigase.xmpp.impl.JabberIqRoster;
import tigase.xmpp.impl.Presence;
import tigase.xmpp.impl.roster.RosterAbstract;

/**
 *
 * @author tskoda
 */
public class InitializeDynamicRosters implements Task<PluginConfigurator> {

    private final IDynamicDomainRoster dynamicRoster;
    private final IPersonalRosterListener personalRosterListener;

    public InitializeDynamicRosters(IDynamicDomainRoster dynamicRoster, IPersonalRosterListener personalRosterListener) {
        this.dynamicRoster = dynamicRoster;
        this.personalRosterListener = personalRosterListener;
    }

    @Override
    public void run(PluginConfigurator config) {
        try {
            config.configure(Presence.PRESENCE_ELEMENT_NAME, IDynamicDomainRoster.class.getName(), this.dynamicRoster);
            config.configure(Presence.PRESENCE_ELEMENT_NAME, "dynamic-roster-classes", DynamicDomainRosterHandle.class.getName());
            config.configure(Presence.PRESENCE_ELEMENT_NAME, JabberIqRoster.GetRosterHandler.class.getName(), this.dynamicRoster);
            
            config.configure(RosterAbstract.XMLNS, IDynamicDomainRoster.class.getName(), this.dynamicRoster);
            //we use handle because dynamic roster gets created with default constructor and can't be configured later on
            config.configure(RosterAbstract.XMLNS, "dynamic-roster-classes", DynamicDomainRosterHandle.class.getName()); 
            config.configure(RosterAbstract.XMLNS, JabberIqRoster.SetRosterHandler.class.getName(), this.personalRosterListener);
            config.configure(RosterAbstract.XMLNS, JabberIqRoster.SetRosterListener.class.getName(), this.personalRosterListener);
            config.configure(RosterAbstract.XMLNS, JabberIqRoster.GetRosterHandler.class.getName(), this.dynamicRoster);
        } catch (TigaseDBException ex) {
            throw new XmppException("Dynamic roster configuration failed", ex, Code.INTERNAL_SERVER_ERROR);
        }
    }

}
