/*
 * Copyright 2013 Kerio Technologies s.r.o. All rights reserved.
 */
package com.kerio.im.tigase.derby;

import com.kerio.im.tigase.derby.version.Version;
import java.text.MessageFormat;

/**
 *
 * @author tskoda
 */
public final class DerbyDatabase {

    public static enum TABLE {

        TIG_USERS("tig_users",
                //create table 
                "create table tig_users ("
                + "            uid BIGINT GENERATED BY DEFAULT AS IDENTITY NOT NULL PRIMARY KEY,"
                + "        user_id varchar(2049) NOT NULL,"
                + "        user_pw varchar(255) default NULL,"
                + "acc_create_time timestamp DEFAULT CURRENT_TIMESTAMP,"
                + "     last_login timestamp,"
                + "    last_logout timestamp,"
                + "  online_status int default 0,"
                + "  failed_logins int default 0,"
                + " account_status int default 1,"
                + "       k_status int default 1,"
                + "          k_gal int default 1)",
                //indexes
                "create unique index user_id on tig_users ( user_id )",
                "create index user_pw on tig_users (user_pw)",
                "create index last_login on tig_users (last_login)",
                "create index last_logout on tig_users (last_logout)",
                "create index account_status on tig_users (account_status)",
                "create index online_status on tig_users (online_status)",
                "create index k_status on tig_users (k_status)"),
        TIG_NODES("tig_nodes",
                //create table 
                "create table tig_nodes ("
                + "       nid BIGINT GENERATED BY DEFAULT AS IDENTITY NOT NULL PRIMARY KEY,"
                + "parent_nid bigint,"
                + "       uid bigint NOT NULL references tig_users(uid),"
                + "      node varchar(255) NOT NULL)",
                //indexes
                "create unique index tnode on tig_nodes ( parent_nid, uid, node )",
                "create index node on tig_nodes ( node )",
                "create index nuid on tig_nodes (uid)",
                "create index parent_nid on tig_nodes (parent_nid)"),
        TIG_PAIRS("tig_pairs",
                //create table
                "create table tig_pairs ("
                + " nid bigint references tig_nodes(nid),"
                + " uid bigint NOT NULL references tig_users(uid),"
                + "pkey varchar(255) NOT NULL,"
                + "pval clob(200 K))",
                //indexes
                "create index pkey on tig_pairs ( pkey )",
                "create index puid on tig_pairs (uid)",
                "create index pnid on tig_pairs (nid)"),
        TIG_SHORT_NEWS("short_news",
                //create table
                "create table short_news ("
                + "           snid BIGINT GENERATED BY DEFAULT AS IDENTITY NOT NULL PRIMARY KEY,"
                + "publishing_time timestamp default current timestamp,"
                + "      news_type varchar(10),"
                + "         author varchar(128) NOT NULL,"
                + "        subject varchar(128) NOT NULL,"
                + "           body varchar(1024) NOT NULL)",
                //indexes
                "create index publishing_time on short_news (publishing_time)",
                "create index author on short_news (author)",
                "create index news_type on short_news (news_type)"),
        XMPP_STANZA("xmpp_stanza",
                //create table
                "create table xmpp_stanza ("
                + "    id BIGINT GENERATED BY DEFAULT AS IDENTITY NOT NULL PRIMARY KEY,"
                + "stanza long varchar NOT NULL)"),
        //added in 5_3
        PERSONAL_ROSTER("personal_roster",
                "create table personal_roster ("
                + "     owner varchar(2049) NOT NULL,"
                + "     buddy varchar(2049) NOT NULL,"
                + "      name varchar(2049),"
                + "    groups varchar(2049),"
                + "      flag varchar(20) NOT NULL,"
                + " primary key (owner, buddy))");
        final String name;
        final String create;
        final String[] indexes;

        private TABLE(String name, String create, String... indexes) {
            this.create = create;
            this.name = name;
            this.indexes = indexes;
        }
    }

    public static enum PROCEDURE {

        GET_DB_PROPERTY("CREATE function TigGetDBProperty(tkey varchar(255)) returns long varchar "
                + "PARAMETER STYLE JAVA "
                + "LANGUAGE JAVA "
                + "READS SQL DATA "
                + "EXTERNAL NAME 'tigase.db.derby.StoredProcedures.tigGetDBProperty' "),
        PUT_DB_PROPERTY("CREATE procedure TigPutDBProperty(tkey varchar(255), tval varchar(32672))  "
                + "PARAMETER STYLE JAVA "
                + "LANGUAGE JAVA "
                + "MODIFIES SQL DATA "
                + "EXTERNAL NAME 'tigase.db.derby.StoredProcedures.tigPutDBProperty' "),
        INIT_DB("CREATE procedure TigInitdb()  "
                + "PARAMETER STYLE JAVA "
                + "LANGUAGE JAVA "
                + "MODIFIES SQL DATA "
                + "EXTERNAL NAME 'tigase.db.derby.StoredProcedures.tigInitdb' "),
        ADD_USER("CREATE procedure TigAddUser(userId varchar(2049), userPw varchar(255))  "
                + "PARAMETER STYLE JAVA "
                + "LANGUAGE JAVA "
                + "MODIFIES SQL DATA "
                + "DYNAMIC RESULT SETS 1 "
                + "EXTERNAL NAME 'tigase.db.derby.StoredProcedures.tigAddUser' "),
        ADD_USER_PLAIN_PW("CREATE procedure TigAddUserPlainPw(userId varchar(2049), userPw varchar(255))  "
                + "PARAMETER STYLE JAVA "
                + "LANGUAGE JAVA "
                + "MODIFIES SQL DATA "
                + "DYNAMIC RESULT SETS 1 "
                + "EXTERNAL NAME 'tigase.db.derby.StoredProcedures.tigAddUserPlainPw' "),
        GET_USER_UID("CREATE procedure TigGetUserDBUid(userId varchar(2049))  "
                + "PARAMETER STYLE JAVA "
                + "LANGUAGE JAVA "
                + "READS SQL DATA "
                + "DYNAMIC RESULT SETS 1 "
                + "EXTERNAL NAME 'tigase.db.derby.StoredProcedures.tigGetUserDBUid' "),
        REMOVE_USER("CREATE procedure TigRemoveUser(userId varchar(2049))  "
                + "PARAMETER STYLE JAVA "
                + "LANGUAGE JAVA "
                + "MODIFIES SQL DATA "
                + "EXTERNAL NAME 'tigase.db.derby.StoredProcedures.tigRemoveUser' "),
        GET_PASSWORD("CREATE procedure TigGetPassword(userId varchar(2049))  "
                + "PARAMETER STYLE JAVA "
                + "LANGUAGE JAVA "
                + "READS SQL DATA "
                + "DYNAMIC RESULT SETS 1 "
                + "EXTERNAL NAME 'tigase.db.derby.StoredProcedures.tigGetPassword' "),
        UPDATE_PASSWORD_PLAIN_REV("CREATE procedure TigUpdatePasswordPlainPwRev(userId varchar(2049), userPw varchar(255)) "
                + "  PARAMETER STYLE JAVA "
                + "  LANGUAGE JAVA "
                + "  MODIFIES SQL DATA "
                + "  EXTERNAL NAME 'tigase.db.derby.StoredProcedures.tigUpdatePasswordPlainPwRev' "),
        UPDATE_PASSWORD_PLAIN("CREATE procedure TigUpdatePasswordPlainPw(userId varchar(2049), userPw varchar(255))  "
                + "PARAMETER STYLE JAVA "
                + "LANGUAGE JAVA "
                + "MODIFIES SQL DATA "
                + "EXTERNAL NAME 'tigase.db.derby.StoredProcedures.tigUpdatePasswordPlainPw' "),
        UPDATE_PASSWORD("CREATE procedure TigUpdatePassword(userId varchar(2049), userPw varchar(255))  "
                + "PARAMETER STYLE JAVA "
                + "LANGUAGE JAVA "
                + "MODIFIES SQL DATA "
                + "EXTERNAL NAME 'tigase.db.derby.StoredProcedures.tigUpdatePassword' "),
        ONLINE_USERS("CREATE procedure TigOnlineUsers()  "
                + "PARAMETER STYLE JAVA "
                + "LANGUAGE JAVA "
                + "READS SQL DATA "
                + "DYNAMIC RESULT SETS 1 "
                + "EXTERNAL NAME 'tigase.db.derby.StoredProcedures.tigOnlineUsers' "),
        OFFLINE_USERS("CREATE procedure TigOfflineUsers()  "
                + "PARAMETER STYLE JAVA "
                + "LANGUAGE JAVA "
                + "READS SQL DATA "
                + "DYNAMIC RESULT SETS 1 "
                + "EXTERNAL NAME 'tigase.db.derby.StoredProcedures.tigOfflineUsers' "),
        ALL_USERS("CREATE procedure TigAllUsers()  "
                + "PARAMETER STYLE JAVA "
                + "LANGUAGE JAVA "
                + "READS SQL DATA "
                + "DYNAMIC RESULT SETS 1 "
                + "EXTERNAL NAME 'tigase.db.derby.StoredProcedures.tigAllUsers' "),
        ALL_USERS_COUNT("CREATE procedure TigAllUsersCount()  "
                + "PARAMETER STYLE JAVA "
                + "LANGUAGE JAVA "
                + "READS SQL DATA "
                + "DYNAMIC RESULT SETS 1 "
                + "EXTERNAL NAME 'tigase.db.derby.StoredProcedures.tigAllUsersCount' "),
        LOGIN_PLAIN_PW("CREATE procedure TigUserLoginPlainPw(userId varchar(2049), userPw varchar(255))  "
                + "PARAMETER STYLE JAVA "
                + "LANGUAGE JAVA "
                + "MODIFIES SQL DATA "
                + "DYNAMIC RESULT SETS 1 "
                + "EXTERNAL NAME 'tigase.db.derby.StoredProcedures.tigUserLoginPlainPw' "),
        USER_LOGIN("CREATE procedure TigUserLogin(userId varchar(2049), userPw varchar(255))  "
                + "PARAMETER STYLE JAVA "
                + "LANGUAGE JAVA "
                + "MODIFIES SQL DATA "
                + "DYNAMIC RESULT SETS 1 "
                + "EXTERNAL NAME 'tigase.db.derby.StoredProcedures.tigUserLogin' "),
        USER_LOGOUT("CREATE procedure TigUserLogout(userId varchar(2049))  "
                + "PARAMETER STYLE JAVA "
                + "LANGUAGE JAVA "
                + "MODIFIES SQL DATA "
                + "EXTERNAL NAME 'tigase.db.derby.StoredProcedures.tigUserLogout' "),
        DISABLE_ACCOUNT("CREATE procedure TigDisableAccount(userId varchar(2049))  "
                + "PARAMETER STYLE JAVA "
                + "LANGUAGE JAVA "
                + "MODIFIES SQL DATA "
                + "EXTERNAL NAME 'tigase.db.derby.StoredProcedures.tigDisableAccount' "),
        ENABLE_ACOUNT("CREATE procedure TigEnableAccount(userId varchar(2049))  "
                + "PARAMETER STYLE JAVA "
                + "LANGUAGE JAVA "
                + "MODIFIES SQL DATA "
                + "EXTERNAL NAME 'tigase.db.derby.StoredProcedures.tigEnableAccount' "),
        ACTIVE_ACCOUNTS("CREATE procedure TigActiveAccounts()  "
                + "PARAMETER STYLE JAVA "
                + "LANGUAGE JAVA "
                + "READS SQL DATA "
                + "DYNAMIC RESULT SETS 1 "
                + "EXTERNAL NAME 'tigase.db.derby.StoredProcedures.tigActiveAccounts' "),
        DISABLED_ACCOUNTS("CREATE procedure TigDisabledAccounts()  "
                + "PARAMETER STYLE JAVA "
                + "LANGUAGE JAVA "
                + "READS SQL DATA "
                + "DYNAMIC RESULT SETS 1 "
                + "EXTERNAL NAME 'tigase.db.derby.StoredProcedures.tigDisabledAccounts' "),
        ADD_NODE("CREATE procedure TigAddNode(parentNid bigint, uid bigint, node varchar(255))  "
                + "PARAMETER STYLE JAVA "
                + "LANGUAGE JAVA "
                + "MODIFIES SQL DATA "
                + "DYNAMIC RESULT SETS 1 "
                + "EXTERNAL NAME 'tigase.db.derby.StoredProcedures.tigAddNode' "),
        UPDATE_PAIRS("CREATE procedure TigUpdatePairs(nid bigint, uid bigint, tkey varchar(255), tval clob(200 K))   "
                + "PARAMETER STYLE JAVA "
                + "LANGUAGE JAVA "
                + "MODIFIES SQL DATA "
                + "EXTERNAL NAME 'tigase.db.derby.StoredProcedures.tigUpdatePairs' "),
        K_ADD_USER("CREATE procedure kAddUser(userId varchar(2049), userPw varchar(255), enabled int, visible int)  "
                + "PARAMETER STYLE JAVA "
                + "LANGUAGE JAVA "
                + "MODIFIES SQL DATA "
                + "DYNAMIC RESULT SETS 1 "
                + "EXTERNAL NAME 'com.kerio.im.tigase.derby.StoredProcedure.kAddUser' ");

        final String create;

        private PROCEDURE(String create) {
            this.create = create;
        }
    }

    public static enum DATA {

        DB_PROPERTIES("call TigAddUserPlainPw('db-properties', NULL)"),
        DB_SCHEMA("call TigPutDBProperty('schema-version', '" + Version.CURRENT + "')");
        final String insert;

        private DATA(String insert) {
            this.insert = insert;
        }
    }
    
    public static enum SCHEMA {
        UPDATE_DB_SCHEMA("call TigPutDBProperty(''schema-version'', ''{0}'')");
        
        private final String sql;

        private SCHEMA(String update) {
            this.sql = update;
        }
        
        String create(String version) {
            return MessageFormat.format(sql, version);
        }
        
    }

    private DerbyDatabase() {
    }
}
