/*
 * Copyright 2013 Kerio Technologies s.r.o. All rights reserved.
 */
package com.kerio.im.tigase.derby.version;

/**
 *
 * @author tskoda
 */
public enum Version {

    /**
     * Points to undefined schema.
     */
    UNDEFINED(0, 0),
    /**
     * Original scheme version.
     */
    V_5_1(5, 1),
    /**
     * Changed TIG_PAIRS.PVAL type to CLOB (long rosters) Added TIG_USERS.K_GAL
     * and TIG_USERS.K_STATUS columns. This is the first public version so upgrades
     * from older schemas are not implemented.
     */
    V_5_2(5, 2),
    /**
     * Added TIG_PERSONAL_ROSTER table
     */
    V_5_3(5, 3),
    /**
     * Removes all offline messages because of re-delivery see #CONNECT-41692
     */
    V_5_4(5, 4),
    /**
     * Points to last schema.
     */
    CURRENT(V_5_4);
    final int major;
    final int minor;

    private Version(int major, int minor) {
        this.major = major;
        this.minor = minor;
    }

    private Version(Version version) {
        this.major = version.major;
        this.minor = version.minor;
    }

    /**
     * Returns CURRENT if found version corresponds to it.
     * @param version
     * @return 
     */
    public static Version fromString(String version) {
        if (version == null) {
            throw new IllegalArgumentException("Unknown schema version");
        }
        int index = version.indexOf('.');
        String[] split = version.split("\\.", 2);

        if (split.length != 2) {
            throw new IllegalArgumentException("Unknown schema version " + version);
        }

        Version result = Version.UNDEFINED;
        try {
            int major = Integer.valueOf(split[0]);
            int minor = Integer.valueOf(split[1]);


            for (Version v : Version.values()) {
                if ((v.major == major) && (v.minor == minor)) {
                    if (v.isCurrent()) {
                        result = CURRENT;
                    } else {
                        result = v;
                    }
                    break;
                }
            }

        } catch (NumberFormatException e) {
            throw new IllegalArgumentException("Illegal schema version " + version);
        }

        return result;
    }
    
    public boolean isVersion(Version version) {
        return ((this.major == version.major) && (this.minor == version.minor));
    }
    
    public boolean isCurrent() {
        return isVersion(CURRENT);
    }

    @Override
    public String toString() {
        return major + "." + minor;
    }
}
