/*
 *  Copyright 2013 Kerio Technologies s.r.o. All rights reserved.
 */

package com.kerio.im.tigase.roster;

import com.kerio.im.tigase.user.XmppUsersStorePool;
import java.util.HashMap;
import java.util.Map;
import java.util.logging.Level;
import java.util.logging.Logger;
import tigase.xmpp.BareJID;

/**
 *
 * @author tskoda
 */
public class DefaultRoster {
    
    private static final Logger LOG = Logger.getLogger(DefaultRoster.class.getName());
    
    private final XmppUsersStorePool storePool;
    private final Map<String, RosterContent> defaultRosters;
    private final ElementMapper elementMapper;
    private final int maximum;
    
    public DefaultRoster(XmppUsersStorePool storePool, ElementMapper elementMapper, int maximum) {
	this(storePool, new HashMap<String, RosterContent>(), elementMapper, maximum);
    }

    public DefaultRoster(XmppUsersStorePool storePool, Map<String, RosterContent> defaultRosters, ElementMapper elementMapper, int maximum) {
	this.storePool = storePool;
	this.defaultRosters = defaultRosters;
	this.elementMapper = elementMapper;
        this.maximum = maximum;
    }
    
    public RosterContent roster(String domain) {
	if (! this.defaultRosters.containsKey(domain)) {
	    synchronized(this.defaultRosters) {
		if (! this.defaultRosters.containsKey(domain)) {
		    this.defaultRosters.put(domain, newRosterContent(domain));
                    LOG.log(
                            Level.INFO, 
                            "Initializing default roster for domain {0} successful, number of users {1}", 
                            new Object[]{domain, this.defaultRosters.get(domain).size()});
		}
	    }
	}
	
	return this.defaultRosters.get(domain);
    }
    
    private RosterContent newRosterContent(String domain) {
        Map<BareJID, RosterContent.Buddy> content = new HashMap<BareJID, RosterContent.Buddy>();
        boolean maximumAnnounce = this.storePool.get(domain).all(content, this.elementMapper, new EnabledVisibleFilter(), this.maximum + 1);
        
        //content contains owner + content of his roster, that is why we use here maximum + 1
        if (maximumAnnounce) {
            LOG.log(
                    Level.INFO, 
                    "Default Instant Messaging contact list for domain {0} truncated, maximum number of users {1} reached", 
                    new Object[]{domain, this.maximum});
            
            RosterContent.warnAboutMaximumReached(maximum, domain);
        }
        
	return new RosterContent(content, this.elementMapper, this.maximum, maximumAnnounce);
    }
}
