/*
 *  Copyright 2013 Kerio Technologies s.r.o. All rights reserved.
 */
package com.kerio.im.tigase.roster;

import com.kerio.im.bean.User;
import com.kerio.im.core.api.service.GroupService;
import com.kerio.im.tigase.user.UserMapper;
import com.kerio.im.tigase.user.XmppUsersStorePool;
import com.kerio.im.xmpp.api.XmppUsers;
import java.util.Collections;
import java.util.EnumSet;
import java.util.List;
import java.util.Map;
import tigase.db.TigaseDBException;
import tigase.xml.Element;
import tigase.xmpp.BareJID;
import tigase.xmpp.JID;
import tigase.xmpp.NotAuthorizedException;
import tigase.xmpp.XMPPResourceConnection;
import tigase.xmpp.impl.roster.RosterAbstract;
import tigase.xmpp.impl.roster.RosterElement;

/**
 *
 * @author tskoda
 */
public class RosterFactory {

    private final int limit;

    public RosterFactory(int limit) {
        this.limit = limit;
    }

    private boolean isEnabled() {
        return this.limit > 0;
    }

    public ElementMapper newElementMapper(GroupService groupService) {
        return new ElementMapper(groupService, new UserMapper());
    }

    public IDefaultRosterListener newDefaultRosterListener(IDynamicDomainRoster dynamicDomainRoster, RosterNotify rosterNotify, GroupService groupService) {
        if (isEnabled()) {
            return new DefaultRosterListener(dynamicDomainRoster.getDefaultRoster(), dynamicDomainRoster.getPersonalRoster(), rosterNotify, groupService);
        } else {
            return new IDefaultRosterListener() {

                @Override
                public void add(User userToAdd) {
                }

                @Override
                public void remove(User userToRemove) {
                }

                @Override
                public void update(User user, EnumSet<XmppUsers.Changes> changes) {
                }

                @Override
                public void check(User user, Map<BareJID, RosterElement> roster) {
                }

                @Override
                public void repair(User user, Map<BareJID, RosterElement> roster) {
                }

                @Override
                public void bye() {
                }

            };
        }
    }

    public IDynamicDomainRoster newDynamicDomainRoster(XmppUsersStorePool storePool, ElementMapper elementMapper) {
        if (isEnabled()) {

            return new DynamicDomainRoster(
                    new DefaultRoster(storePool, elementMapper, this.limit),
                    new PersonalRoster(storePool),
                    new SessionUtil(),
                    elementMapper);
        } else {

            return new IDynamicDomainRoster() {

                @Override
                public PersonalRoster getPersonalRoster() {
                    return null;
                }

                @Override
                public DefaultRoster getDefaultRoster() {
                    return null;
                }

                @Override
                public void setItemExtraData(Element item) {
                }

                @Override
                public Element getItemExtraData(Element item) {
                    return null;
                }

                @Override
                public void init(Map<String, Object> props) {
                }

                @Override
                public void init(String par) {
                }

                @Override
                public JID[] getBuddies(XMPPResourceConnection session) throws NotAuthorizedException {
                    return new JID[0];
                }

                @Override
                public Element getBuddyItem(XMPPResourceConnection session, JID buddy) throws NotAuthorizedException {
                    return null;
                }

                @Override
                public List<Element> getRosterItems(XMPPResourceConnection session) throws NotAuthorizedException {
                    return Collections.EMPTY_LIST;
                }

                @Override
                public RosterAbstract.SubscriptionType getBuddySubscription(XMPPResourceConnection session, JID buddy) throws NotAuthorizedException, TigaseDBException {
                    return RosterAbstract.SubscriptionType.none;
                }

                @Override
                public boolean manages(XMPPResourceConnection session, JID buddy) throws NotAuthorizedException, TigaseDBException {
                    return false;
                }
            };
        }
    }

    public IPersonalRosterListener newPersonalRosterListener(IDynamicDomainRoster dynamicDomainRoster, GroupService groupService) {

        if (isEnabled()) {
            return new PersonalRosterListener(dynamicDomainRoster.getPersonalRoster(), dynamicDomainRoster.getDefaultRoster(), groupService);
        } else {
            return new IPersonalRosterListener() {

                @Override
                public void removeBuddy(XMPPResourceConnection session, JID buddy) throws NotAuthorizedException, TigaseDBException {
                }

                @Override
                public void addBuddy(XMPPResourceConnection session, JID buddy, String name, String[] gr) throws NotAuthorizedException, TigaseDBException {
                }

                @Override
                public void setBuddySubscription(XMPPResourceConnection session, RosterAbstract.SubscriptionType subscriptionType, JID buddy) throws NotAuthorizedException, TigaseDBException {
                }

                @Override
                public void added(XMPPResourceConnection session, JID buddy) throws NotAuthorizedException, TigaseDBException {
                }

                @Override
                public void removed(XMPPResourceConnection session, JID buddy) throws NotAuthorizedException, TigaseDBException {
                }
            };
        }

    }
}
