/*
 *  Copyright 2013 Kerio Technologies s.r.o. All rights reserved.
 */
package com.kerio.im.tigase.user;

import com.kerio.im.common.error.Code;
import com.kerio.im.tigase.DomainServiceImpl;
import com.kerio.im.tigase.SessionProvider;
import com.kerio.im.xmpp.api.XmppException;
import java.sql.SQLException;
import java.util.HashMap;
import java.util.Map;
import java.util.logging.Level;
import java.util.logging.Logger;
import org.apache.derby.jdbc.EmbeddedDataSource40;

/**
 *
 * @author tskoda
 */
public class XmppUsersStorePool {

    public static final int AVERAGE_NUMBER_OF_DOMAINS = 1;
    
    private static final Logger LOG = Logger.getLogger(XmppUsersStorePool.class.getName());

    private final Map<String, XmppUsersStore> xmppUsersCache;
    private DomainServiceImpl domainService;
    private SessionProvider sessions;

    public XmppUsersStorePool() {
	this.xmppUsersCache = new HashMap<String, XmppUsersStore>();
    }

    public void setDomainService(DomainServiceImpl domainService) {
        this.domainService = domainService;
    }

    public void setSessions(SessionProvider sessions) {
        this.sessions = sessions;
    }
    
    public XmppUsersStore get(String domain) {
	XmppUsersStore result;
	synchronized (this.xmppUsersCache) {
	    try {
		if (this.xmppUsersCache.containsKey(domain)) {
		    result = this.xmppUsersCache.get(domain);
		} else {
		    EmbeddedDataSource40 ds = new EmbeddedDataSource40();
		    ds.setDatabaseName(this.domainService.getDbsPath(domain));
		    result = new XmppUsersStore(domain, ds, this.sessions);
		    this.xmppUsersCache.put(domain, result);
		}
	    } catch (SQLException e) {
		throw new XmppException("Can not get access to database for domain {0}", e, Code.RESOURCE_ERROR, domain);
	    }
	}

	return result;
    }
    
    public void close() {
        for (Map.Entry<String, XmppUsersStore> store: this.xmppUsersCache.entrySet()) {
            store.getValue().bye();
            LOG.log(Level.INFO, "Closing access to {0}", store.getKey());
        }
    }
}
