/*
 * Copyright 2014 Kerio Technologies s.r.o. All rights reserved
 */
package com.kerio.im.tigase;

import static org.hamcrest.CoreMatchers.is;
import static org.junit.Assert.assertThat;
import org.junit.Test;
import tigase.xmpp.JID;
import tigase.xmpp.StanzaType;
import tigase.xmpp.XMPPResourceConnection;
import tigase.xmpp.XMPPSession;
import static com.kerio.im.tigase.ResourceSelectorTestUtil.*;
import org.junit.Rule;
import org.junit.rules.ExpectedException;

/**
 *
 * @author tskoda
 */
public class MessageResourceSelectorWithResourceMatchTest {

    @Rule
    public ExpectedException exception = ExpectedException.none();
    private MessageResourceSelector selector = new MessageResourceSelector();
    private JID jid = JID.jidInstanceNS("user@test/resource");

    @Test
    public void testEmpty_matchNegative() throws Exception {
        StanzaType type = null; // null == normal message
        XMPPResourceConnection active = with(-10, MessageResourceSelector.Show.availible);
        XMPPSession mockSession = mockSession(
                jid,
                0,
                active,
                with(100, MessageResourceSelector.Show.availible),
                with(1000, MessageResourceSelector.Show.chat),
                with(1, MessageResourceSelector.Show.dnd));

        exception.expect(serviceUnavailible());
        selector.preferredResources(jid, type, mockSession);
    }

    @Test
    public void testNormal_matchNegative() throws Exception {
        StanzaType type = StanzaType.normal;
        XMPPResourceConnection active = with(-10, MessageResourceSelector.Show.availible);
        XMPPSession mockSession = mockSession(
                jid,
                0,
                active,
                with(100, MessageResourceSelector.Show.availible),
                with(1000, MessageResourceSelector.Show.chat),
                with(1, MessageResourceSelector.Show.dnd));

        exception.expect(serviceUnavailible());
        selector.preferredResources(jid, type, mockSession);
    }

    @Test
    public void testChat_matchNegative_one() throws Exception {
        StanzaType type = StanzaType.chat;
        XMPPResourceConnection active = with(-10, MessageResourceSelector.Show.availible);
        XMPPSession mockSession = mockSession(
                jid,
                0,
                active);

        assertThat(selector.preferredResources(jid, type, mockSession).size(), is(0));
    }

    @Test
    public void testChat_matchNegative_allNegative() throws Exception {
        StanzaType type = StanzaType.chat;
        XMPPResourceConnection active = with(-10, MessageResourceSelector.Show.availible);
        XMPPSession mockSession = mockSession(
                jid,
                0,
                active,
                with(-100, MessageResourceSelector.Show.availible),
                with(-1000, MessageResourceSelector.Show.chat),
                with(-1, MessageResourceSelector.Show.dnd));

        assertThat(selector.preferredResources(jid, type, mockSession).size(), is(0)); //offline delivery
    }

    @Test
    public void testChat_matchNegative_allPositive() throws Exception {
        StanzaType type = StanzaType.chat;
        XMPPResourceConnection active = with(-10, MessageResourceSelector.Show.availible);
        XMPPSession mockSession = mockSession(
                jid,
                0,
                active,
                with(100, MessageResourceSelector.Show.availible),
                with(1000, MessageResourceSelector.Show.chat),
                with(1, MessageResourceSelector.Show.dnd));

        assertThat(selector.preferredResources(jid, type, mockSession).size(), is(1)); //selected resource delivery
    }

    @Test
    public void testGroupchat_matchNegative() throws Exception {
        StanzaType type = StanzaType.groupchat;
        XMPPResourceConnection active = with(-10, MessageResourceSelector.Show.availible);
        XMPPSession mockSession = mockSession(
                jid,
                0,
                active,
                with(100, MessageResourceSelector.Show.availible),
                with(1000, MessageResourceSelector.Show.chat),
                with(1, MessageResourceSelector.Show.dnd));

        exception.expect(serviceUnavailible());
        selector.preferredResources(jid, type, mockSession);
    }

    @Test
    public void testHeadline_matchNegative() throws Exception {
        StanzaType type = StanzaType.headline;
        XMPPResourceConnection active = with(-10, MessageResourceSelector.Show.availible);
        XMPPSession mockSession = mockSession(
                jid,
                0,
                active,
                with(100, MessageResourceSelector.Show.availible),
                with(1000, MessageResourceSelector.Show.chat),
                with(1, MessageResourceSelector.Show.dnd));

        exception.expect(serviceUnavailible());
        selector.preferredResources(jid, type, mockSession);
    }

    @Test
    public void testEmpty_matchPositive() throws Exception {
        StanzaType type = null; // null == normal message
        XMPPResourceConnection active = with(10, MessageResourceSelector.Show.away);
        XMPPSession mockSession = mockSession(
                jid,
                0,
                active,
                with(100, MessageResourceSelector.Show.availible),
                with(1000, MessageResourceSelector.Show.chat),
                with(1, MessageResourceSelector.Show.dnd));

        assertThat(selector.preferredResources(jid, type, mockSession), containsOnly(active));
    }

    @Test
    public void testNormal_matchPositive() throws Exception {
        StanzaType type = StanzaType.normal;
        XMPPResourceConnection active = with(10, MessageResourceSelector.Show.away);
        XMPPSession mockSession = mockSession(
                jid,
                0,
                active,
                with(100, MessageResourceSelector.Show.availible),
                with(1000, MessageResourceSelector.Show.chat),
                with(1, MessageResourceSelector.Show.dnd));

        assertThat(selector.preferredResources(jid, type, mockSession), containsOnly(active));
    }

    @Test
    public void testChat_matchPositive() throws Exception {
        StanzaType type = StanzaType.chat;
        XMPPResourceConnection active = with(10, MessageResourceSelector.Show.away);
        XMPPSession mockSession = mockSession(
                jid,
                0,
                active,
                with(100, MessageResourceSelector.Show.availible),
                with(1000, MessageResourceSelector.Show.chat),
                with(1, MessageResourceSelector.Show.dnd));

        assertThat(selector.preferredResources(jid, type, mockSession), containsOnly(active));
    }

    @Test
    public void testGroupchat_matchPositive() throws Exception {
        StanzaType type = StanzaType.groupchat;
        XMPPResourceConnection active = with(10, MessageResourceSelector.Show.away);
        XMPPSession mockSession = mockSession(
                jid,
                0,
                active,
                with(100, MessageResourceSelector.Show.availible),
                with(1000, MessageResourceSelector.Show.chat),
                with(1, MessageResourceSelector.Show.dnd));

        assertThat(selector.preferredResources(jid, type, mockSession), containsOnly(active));
    }

    @Test
    public void testHeadline_matchPositive() throws Exception {
        StanzaType type = StanzaType.headline;
        XMPPResourceConnection active = with(10, MessageResourceSelector.Show.away);
        XMPPSession mockSession = mockSession(
                jid,
                0,
                active,
                with(100, MessageResourceSelector.Show.availible),
                with(1000, MessageResourceSelector.Show.chat),
                with(1, MessageResourceSelector.Show.dnd));

        assertThat(selector.preferredResources(jid, type, mockSession), containsOnly(active));
    }

}
