/*
 * Copyright 2013 Kerio Technologies s.r.o. All rights reserved.
 */
package com.kerio.im.tigase.derby;

import java.sql.Connection;
import java.sql.ResultSet;
import java.sql.SQLException;
import java.sql.Statement;
import org.apache.derby.jdbc.EmbeddedDataSource40;

/**
 *
 * @author tskoda
 */
public class DerbyUtilCommon {

    public static void init(Connection connection, DerbyDatabase.TABLE[] tables, DerbyDatabase.PROCEDURE[] procedures, DerbyDatabase.DATA[] data) throws SQLException {

        Statement statement = null;

        try {
            statement = connection.createStatement();

            if (tables != null) {
                for (DerbyDatabase.TABLE table : tables) {
                    statement.executeUpdate(table.create);
                    for (String index : table.indexes) {
                        statement.executeUpdate(index);
                    }
                }
            }

            if (procedures != null) {
                for (DerbyDatabase.PROCEDURE procedure : procedures) {
                    statement.executeUpdate(procedure.create);
                }
            }

            if (data != null) {
                for (DerbyDatabase.DATA element : data) {
                    statement.execute(element.insert);
                }
            }
        } finally {
            close(statement);
        }
    }
    
    public static void init(Connection connection, String... sql) throws SQLException {

        Statement statement = null;

        try {
            statement = connection.createStatement();

            if (sql != null) {
                for (String element : sql) {
                    statement.execute(element);
                }
            }
        } finally {
            close(statement);
        }
    }

    public static void close(ResultSet rs) {
        if (rs != null) {
            try {
                rs.close();
            } catch (Exception e) {
                System.err.println("Can not close Statement");
            }
        }
    }

    public static void close(Statement statement) {
        if (statement != null) {
            try {
                statement.close();
            } catch (Exception e) {
                System.err.println("Can not close Statement");
            }
        }
    }

    public static void close(Connection connection) {
        if (connection != null) {
            try {
                connection.close();
            } catch (Exception e) {
                System.err.println("Can not close connection");
            }
        }
    }

    public static void shutdown(EmbeddedDataSource40 ds) {

        try {
            ds.getConnection();
            System.err.println("Can not destroy test database: " + ds.toString());
        } catch (SQLException ex) {
            String engineShutdownSucc = "XJ015";
            String databaseShutdownSucc = "08006";
            if (engineShutdownSucc.equals(ex.getSQLState()) || (databaseShutdownSucc.equals(ex.getSQLState()))) {
                //ok
            } else {
                System.err.println("Can not destroy test database: " + ds.toString());
            }
        } catch (Exception ex) {
            System.err.println("Can not destroy test database: " + ds.toString());
        }

    }
}
