/*
 * Copyright 2013 Kerio Technologies s.r.o. All rights reserved.
 */
package com.kerio.im.tigase.derby;

import java.io.File;
import java.io.FileFilter;
import java.io.IOException;
import java.sql.Connection;
import java.sql.SQLException;
import java.util.Arrays;
import org.apache.derby.jdbc.EmbeddedDataSource40;

/**
 *
 * @author tskoda
 */
public class DerbyUtilFile {

    public static File createTempFile(File folder) throws IOException {
        return File.createTempFile("test", "tmp", folder);
    }

    public static File createTempFolder(File folder) throws IOException {
        File temp;

        temp = File.createTempFile("db_", Long.toString(System.nanoTime()), folder);

        if (!(temp.delete())) {
            throw new IOException("Could not delete temp file: " + temp.getAbsolutePath());
        }

        if (!(temp.mkdir())) {
            throw new IOException("Could not create temp directory: " + temp.getAbsolutePath());
        }
        return temp;
    }

    public static File createTempFolder() throws IOException {
        return createTempFolder(null);
    }

    public static void createDatabase(File db) throws SQLException {
        EmbeddedDataSource40 toCreate = new EmbeddedDataSource40();
        toCreate.setDatabaseName(db.getAbsolutePath());
        toCreate.setCreateDatabase("create");
        
        DerbyUtilCommon.close(toCreate.getConnection());

        shutdown(db);
    }

    public static void shutdown(File db) {
        EmbeddedDataSource40 ds = new EmbeddedDataSource40();
        ds.setDatabaseName(db.getAbsolutePath());
        ds.setShutdownDatabase("shutdown");
        DerbyUtilCommon.shutdown(ds);
    }

    /**
     *
     * @param derbyFolder
     */
    public static void deleteDerbyFolder(final File derbyFolder) {
        deleteFolderWithFiles(new File(derbyFolder, "log"));
        deleteFolderWithFiles(new File(derbyFolder, "seg0"));
        deleteFolderWithFiles(new File(derbyFolder, "tmp"));
        deleteFolderWithFiles(derbyFolder);
    }

    public static void deleteFolderWithFiles(final File tempFolder) {
        if ((tempFolder == null) || (!tempFolder.exists()) || (!tempFolder.isDirectory())) {
            return;
        }

        File[] subfolders = tempFolder.listFiles(new FileFilter() {
            @Override
            public boolean accept(File pathname) {
                return pathname.isDirectory();
            }
        });

        if (subfolders.length > 0) {
            System.err.println("Not going to delete folder with unexpected subfolders, subfolders: " + Arrays.toString(subfolders));
        }

        /**/
        for (File child : tempFolder.listFiles()) {
            if (!child.delete()) {
                System.err.println("File " + child.getAbsolutePath() + " was not deleted");
            }
        }

        tempFolder.delete();
    }
}
