/*
 *  Copyright 2013 Kerio Technologies s.r.o. All rights reserved.
 */

package com.kerio.im.tigase.user;

import com.kerio.im.bean.User;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.List;
import tigase.xmpp.BareJID;
import tigase.xmpp.JID;

/**
 *
 * @author tskoda
 */
public class UserUtil {

    public User user(int i) {
        return user(i, true, true);
    }

    public User renamed(User u, String newName) {
        return new User(u.getLogin(), u.getDomain(), newName, u.isEnabled(), u.isVisible(), u.isAllowedSendOutside());
    }

    public User disabled(int i) {
        return user(i, false, true);
    }

    public User enabled(User u) {
        return new User(u.getLogin(), u.getDomain(), u.getFullName(), true, u.isVisible(), u.isAllowedSendOutside());
    }

    public User disabled(User u) {
        return new User(u.getLogin(), u.getDomain(), u.getFullName(), false, u.isVisible(), u.isAllowedSendOutside());
    }

    public User invisible(int i) {
        return user(i, true, false);
    }

    public User invisible(User u) {
        return new User(u.getLogin(), u.getDomain(), u.getFullName(), u.isEnabled(), false, u.isAllowedSendOutside());
    }

    protected User visible(User u) {
        return new User(u.getLogin(), u.getDomain(), u.getFullName(), u.isEnabled(), true, u.isAllowedSendOutside());
    }

    public User disabledAndInvisible(int i) {
        return user(i, false, false);
    }

    protected User disabledAndInvisible(User u) {
        return new User(u.getLogin(), u.getDomain(), u.getFullName(), false, false, u.isAllowedSendOutside());
    }

    public User user(int i, boolean status, boolean visible) {
        return new User(XmppUsersStoreAbstract.USER_PREFIX + i, XmppUsersStoreAbstract.DOMAIN, "User " + i, status, visible, false);
    }

    public BareJID bare(int i) {
        return BareJID.bareJIDInstanceNS(XmppUsersStoreAbstract.USER_PREFIX + i, XmppUsersStoreAbstract.DOMAIN);
    }

    public BareJID bare(User u) {
        return BareJID.bareJIDInstanceNS(u.getLogin(), u.getDomain());
    }

    public JID jid(int user, int i) {
        return jid(bare(user), i);
    }

    public JID jid(BareJID jid, int i) {
        return JID.jidInstanceNS(jid, "testresource-" + i);
    }
    
    public JID jid(BareJID jid) {
        return JID.jidInstance(jid);
    }
    
    public JID jid(int i) {
        return JID.jidInstance(bare(i));
    }
    
    public JID jid(User u) {
        return jid(bare(u));
    }
    
    public BareJID domain(int i) {
        return BareJID.bareJIDInstanceNS(XmppUsersStoreAbstract.USER_PREFIX + i, XmppUsersStoreAbstract.DOMAIN + "_another");
    }
    
    public User domain(int user, String domain) {
        User defDomainUser = user(user);
        return new User(defDomainUser.getLogin(), domain, defDomainUser.getFullName(), defDomainUser.isEnabled(), defDomainUser.isVisible(), defDomainUser.isAllowedSendOutside());
    }

    /**
     * Returns all users, no need to filter out disabled and indivisible and
     * the owner
     *
     * @param users
     * @param except
     * @return
     */
    public List<User> roster(User[] users, int except) {
        List<User> result = new ArrayList();
        for (int i = 0; i < users.length; i++) {
            if (i != except) {
                result.add(users[i]);
            }
        }
        return result;
    }

    public User[] roster(User[] users, User... additional) {
        List<User> allList = new ArrayList();
        allList.addAll(Arrays.asList(users));
        List<User> additionalList = Arrays.asList(additional);
        allList.addAll(additionalList);
        return allList.toArray(new User[allList.size()]);
    }
    
}
