/*
 *  Copyright 2013 Kerio Technologies s.r.o. All rights reserved.
 */
package com.kerio.im.tigase.user;

import com.kerio.im.tigase.derby.DerbyUtilMemory;
import com.kerio.im.tigase.roster.BuddyUtil;
import com.kerio.im.tigase.roster.PersonalRoster;
import static com.kerio.im.tigase.user.XmppUsersStoreAbstract.DOMAIN;
import java.sql.SQLException;
import java.util.Map;
import org.junit.Test;
import static org.junit.Assert.*;
import static org.hamcrest.CoreMatchers.*;
import tigase.xmpp.BareJID;

/**
 *
 * @author tskoda
 */
public class XmppUsersStorePersonalRosterTest extends XmppUsersStoreAbstract {

    private static final BuddyUtil bd = new BuddyUtil();

    private XmppUsersStore verifyStore;

    @Override
    public void before() throws SQLException {
        super.before();
        this.verifyStore = new XmppUsersStore(
                DOMAIN,
                DerbyUtilMemory.createDS(DOMAIN),
                this.sessions);
    }

    @Test
    public void testGetAllEmpty() {
        assertThat(this.store.personalAll(as.bare(1)).size(), is(0));
    }

    @Test
    public void testGetAll() {
        this.store.personalUpdate(as.bare(1), new PersonalRoster.Buddy(as.bare(2), "name2", new String[]{"group2"}, PersonalRoster.Buddy.FLAG.updated));

        assertThat(this.verifyStore.personalAll(as.bare(1)).size(), is(1));
        assertThat(this.verifyStore.personalAll(as.domain(1)).size(), is(0));
        assertThat(this.verifyStore.personalAll(as.bare(2)).size(), is(0));
    }

    @Test
    public void testUpdateInsertOnly() {
        this.store.personalUpdate(as.bare(1), new PersonalRoster.Buddy(as.bare(2), "name2", new String[]{"group2"}, PersonalRoster.Buddy.FLAG.updated));
        this.store.personalUpdate(as.bare(1), new PersonalRoster.Buddy(as.bare(3), "name3", new String[]{"group3", "group4"}, PersonalRoster.Buddy.FLAG.updated));
        this.store.personalUpdate(as.bare(1), new PersonalRoster.Buddy(as.bare(4), null, null, PersonalRoster.Buddy.FLAG.added));
        this.store.personalUpdate(as.bare(1), new PersonalRoster.Buddy(as.bare(5), null, null, PersonalRoster.Buddy.FLAG.deleted));

        Map<BareJID, PersonalRoster.Buddy> all = this.verifyStore.personalAll(as.bare(1));

        assertThat(all.size(), is(4));
        assertThat(all.get(as.bare(2)), allOf(bd.name("name2"), bd.groups("group2"), bd.notManaged()));
        assertThat(all.get(as.bare(3)), allOf(bd.name("name3"), bd.groups("group3", "group4"), bd.notManaged()));
        assertThat(all.get(as.bare(4)), allOf(bd.name(null), bd.groups((String[]) null), bd.added()));
        assertThat(all.get(as.bare(5)), allOf(bd.name(null), bd.groups((String[]) null), bd.removed()));
    }

    @Test
    public void testUpdate() {
        this.store.personalUpdate(as.bare(2), new PersonalRoster.Buddy(as.bare(2), "name2", new String[]{"group2"}, PersonalRoster.Buddy.FLAG.updated));
        this.store.personalUpdate(as.bare(3), new PersonalRoster.Buddy(as.bare(2), "name2", new String[]{"group2"}, PersonalRoster.Buddy.FLAG.updated));
        assertThat(this.verifyStore.personalAll(as.bare(2)).get(as.bare(2)), allOf(bd.name("name2"), bd.groups("group2"), bd.notManaged()));
        assertThat(this.verifyStore.personalAll(as.bare(3)).get(as.bare(2)), allOf(bd.name("name2"), bd.groups("group2"), bd.notManaged()));

        this.store.personalUpdate(as.bare(1), new PersonalRoster.Buddy(as.bare(2), "name2", new String[]{"group2"}, PersonalRoster.Buddy.FLAG.updated));
        assertThat(this.verifyStore.personalAll(as.bare(1)).get(as.bare(2)), allOf(bd.name("name2"), bd.groups("group2")));

        this.store.personalUpdate(as.bare(1), new PersonalRoster.Buddy(as.bare(2), "name22", new String[]{"group2"}, PersonalRoster.Buddy.FLAG.updated));
        assertThat(this.verifyStore.personalAll(as.bare(1)).get(as.bare(2)), allOf(bd.name("name22"), bd.groups("group2")));

        this.store.personalUpdate(as.bare(1), new PersonalRoster.Buddy(as.bare(2), null, new String[]{"group2", "group3"}, PersonalRoster.Buddy.FLAG.updated));
        assertThat(this.verifyStore.personalAll(as.bare(1)).get(as.bare(2)), allOf(bd.name(null), bd.groups("group2", "group3")));

        this.store.personalUpdate(as.bare(1), new PersonalRoster.Buddy(as.bare(2), null, new String[]{"group2", "group3"}, PersonalRoster.Buddy.FLAG.deleted));
        assertThat(this.verifyStore.personalAll(as.bare(1)).get(as.bare(2)), allOf(bd.name(null), bd.groups("group2", "group3"), bd.removed()));

        this.store.personalUpdate(as.bare(1), new PersonalRoster.Buddy(as.bare(2), null, new String[]{"group2", "group3"}, PersonalRoster.Buddy.FLAG.added));
        assertThat(this.verifyStore.personalAll(as.bare(1)).get(as.bare(2)), allOf(bd.name(null), bd.groups("group2", "group3"), bd.added()));

        this.store.personalUpdate(as.bare(1), new PersonalRoster.Buddy(as.bare(2), null, null, PersonalRoster.Buddy.FLAG.updated));
        assertThat(this.verifyStore.personalAll(as.bare(1)).get(as.bare(2)), allOf(bd.name(null), bd.groups((String[]) null), bd.notManaged()));

        //not touched by this test
        assertThat(this.verifyStore.personalAll(as.bare(2)).get(as.bare(2)), allOf(bd.name("name2"), bd.groups("group2"), bd.notManaged()));
        assertThat(this.verifyStore.personalAll(as.bare(3)).get(as.bare(2)), allOf(bd.name("name2"), bd.groups("group2"), bd.notManaged()));
    }

    @Test
    public void testRemoveNotExist() {
        this.store.personalRemove(as.bare(1), as.bare(2));
        this.store.personalRemove(as.bare(1), as.bare(3));

        assertThat(this.verifyStore.personalAll(as.bare(1)).size(), is(0));
    }

    @Test
    public void testRemove() {
        this.store.personalRemove(as.bare(1), as.bare(2));

        this.store.personalUpdate(as.bare(1), new PersonalRoster.Buddy(as.bare(2), null, null, PersonalRoster.Buddy.FLAG.updated));
        this.store.personalUpdate(as.bare(2), new PersonalRoster.Buddy(as.bare(2), null, null, PersonalRoster.Buddy.FLAG.updated));
        this.store.personalUpdate(as.bare(3), new PersonalRoster.Buddy(as.bare(2), null, null, PersonalRoster.Buddy.FLAG.updated));

        assertThat(this.verifyStore.personalAll(as.bare(1)).size(), is(1));
        assertThat(this.verifyStore.personalAll(as.bare(2)).size(), is(1));
        assertThat(this.verifyStore.personalAll(as.bare(3)).size(), is(1));

        this.store.personalRemove(as.bare(1), as.bare(2));

        assertThat(this.verifyStore.personalAll(as.bare(1)).size(), is(0));
        assertThat(this.verifyStore.personalAll(as.bare(2)).size(), is(1));
        assertThat(this.verifyStore.personalAll(as.bare(3)).size(), is(1));
    }

    @Test
    public void testRemoveAllEmpty() {
        this.store.personalRemove(as.bare(1));
        this.store.personalRemove(as.bare(2));

        assertThat(this.verifyStore.personalAll(as.bare(1)).size(), is(0));
        assertThat(this.verifyStore.personalAll(as.bare(2)).size(), is(0));
    }

    @Test
    public void testRemoveAll() {
        this.store.personalUpdate(as.bare(1), new PersonalRoster.Buddy(as.bare(2), null, null, PersonalRoster.Buddy.FLAG.updated));
        this.store.personalUpdate(as.bare(1), new PersonalRoster.Buddy(as.bare(3), "name", null, PersonalRoster.Buddy.FLAG.updated));
        this.store.personalUpdate(as.bare(1), new PersonalRoster.Buddy(as.bare(4), null, new String[]{"group"}, PersonalRoster.Buddy.FLAG.updated));
        this.store.personalUpdate(as.bare(1), new PersonalRoster.Buddy(as.bare(5), null, null, PersonalRoster.Buddy.FLAG.deleted));

        this.store.personalUpdate(as.bare(2), new PersonalRoster.Buddy(as.bare(2), null, null, PersonalRoster.Buddy.FLAG.updated));
        this.store.personalUpdate(as.bare(3), new PersonalRoster.Buddy(as.bare(2), null, null, PersonalRoster.Buddy.FLAG.updated));

        assertThat(this.verifyStore.personalAll(as.bare(1)).size(), is(4));
        assertThat(this.verifyStore.personalAll(as.bare(2)).size(), is(1));
        assertThat(this.verifyStore.personalAll(as.bare(3)).size(), is(1));

        this.store.personalRemove(as.bare(1));

        assertThat(this.verifyStore.personalAll(as.bare(1)).size(), is(0));
        assertThat(this.verifyStore.personalAll(as.bare(2)).size(), is(1));
        assertThat(this.verifyStore.personalAll(as.bare(3)).size(), is(1));
    }

    @Test
    public void testUpdateEmptyGroup() {

        //gets inserted
        this.store.personalUpdate(as.bare(1), new PersonalRoster.Buddy(as.bare(2), "name", new String[]{(String) null}, PersonalRoster.Buddy.FLAG.updated));

        assertThat(this.verifyStore.personalAll(as.bare(1)).size(), is(1));
        assertThat(this.verifyStore.personalAll(as.bare(1)).get(as.bare(2)), allOf(bd.name("name"), bd.groups((String) null), bd.notManaged()));

        //gets updated
        this.store.personalUpdate(as.bare(1), new PersonalRoster.Buddy(as.bare(2), "name1", new String[]{(String) null}, PersonalRoster.Buddy.FLAG.updated));
        assertThat(this.verifyStore.personalAll(as.bare(1)).size(), is(1));
        assertThat(this.verifyStore.personalAll(as.bare(1)).get(as.bare(2)), allOf(bd.name("name1"), bd.groups((String) null), bd.notManaged()));

        this.store.personalUpdate(as.bare(1), new PersonalRoster.Buddy(as.bare(2), "name1", new String[]{(String) null, "Colleagues"}, PersonalRoster.Buddy.FLAG.updated));
        assertThat(this.verifyStore.personalAll(as.bare(1)).size(), is(1));
        assertThat(this.verifyStore.personalAll(as.bare(1)).get(as.bare(2)), allOf(bd.name("name1"), bd.groups((String) null, "Colleagues"), bd.notManaged()));
    }

}
