/*
 *  Copyright 2013 Kerio Technologies s.r.o. All rights reserved.
 */
package com.kerio.im.tigase.user;

import com.kerio.im.bean.User;
import java.sql.SQLException;
import java.util.Map;
import static org.hamcrest.CoreMatchers.is;
import static org.hamcrest.CoreMatchers.allOf;
import org.junit.Test;
import static org.junit.Assert.*;
import tigase.xmpp.BareJID;
import tigase.xmpp.impl.roster.RosterElement;

/**
 *
 * @author tskoda
 */
public class XmppUsersStoreRosterTest extends XmppUsersStoreAbstract {

    @Test
    public void testReadEmptyRoster() {
        User[] users = {as.user(0), as.disabled(1), as.invisible(2), as.disabledAndInvisible(3)};
        repositoryAddAll(users);

        XmppUsersStore.Roster userRoster = this.store.readRoster(users[0]);
        assertThat(userRoster.getOwnerBareJID(), is(as.bare(users[0])));
        assertThat(userRoster.getRoster().isEmpty(), is(true));

        XmppUsersStore.Roster disabledRoster = this.store.readRoster(users[1]);
        assertThat(disabledRoster.getOwnerBareJID(), is(as.bare(users[1])));
        assertThat(disabledRoster.getRoster().isEmpty(), is(true));

        XmppUsersStore.Roster invisibleRoster = this.store.readRoster(users[2]);
        assertThat(invisibleRoster.getOwnerBareJID(), is(as.bare(users[2])));
        assertThat(invisibleRoster.getRoster().isEmpty(), is(true));

        XmppUsersStore.Roster disabledAndInvisibleRoster = this.store.readRoster(users[3]);
        assertThat(disabledAndInvisibleRoster.getOwnerBareJID(), is(as.bare(users[3])));
        assertThat(disabledAndInvisibleRoster.getRoster().isEmpty(), is(true));
    }

    @Test
    public void testReadAndSaveRoster() throws SQLException {
        User user = as.user(0);
        repositoryAddAll(new User[]{user});
        
        XmppUsersStore.Roster userRoster = this.store.readRoster(user);

        assertThat(userRoster.getOwnerBareJID(), is(as.bare(user)));
        assertThat(userRoster.getRoster().isEmpty(), is(true));

        userRoster.getRoster().put(as.bare(1), new RosterElement(as.jid(1, 1), as.user(1).getFullName(), new String[]{"1"}, null));
        userRoster.getRoster().put(as.bare(2), new RosterElement(as.jid(2, 1), as.user(2).getFullName(), new String[]{"2"}, null));
        userRoster.getRoster().put(as.bare(3), new RosterElement(as.jid(3, 1), as.user(3).getFullName(), new String[]{"1"}, null));

        Map<BareJID, RosterElement> readUserRosterBefore = database.roster.of(0);
        assertThat(readUserRosterBefore.size(), is(0));

        this.store.saveRoster(userRoster);

        Map<BareJID, RosterElement> readUserRosterAfter = database.roster.of(0);
        assertThat(readUserRosterAfter.size(), is(3));
        assertThat(readUserRosterAfter.get(as.bare(1)), allOf(re.name(as.user(1).getFullName()), re.group("1")));
        assertThat(readUserRosterAfter.get(as.bare(2)), allOf(re.name(as.user(2).getFullName()), re.group("2")));
        assertThat(readUserRosterAfter.get(as.bare(3)), allOf(re.name(as.user(3).getFullName()), re.group("1")));

    }

    @Test
    public void testReadAndSaveEmptyRoster() throws SQLException {
        User user = as.user(0);
        repositoryAddAll(new User[]{user});
        
        XmppUsersStore.Roster initialRoster = this.store.readRoster(user);
        initialRoster.getRoster().put(as.bare(1), new RosterElement(as.jid(1, 1), as.user(1).getFullName(), new String[]{"1"}, null));
        initialRoster.getRoster().put(as.bare(2), new RosterElement(as.jid(2, 1), as.user(2).getFullName(), new String[]{"2"}, null));
        initialRoster.getRoster().put(as.bare(3), new RosterElement(as.jid(3, 1), as.user(3).getFullName(), new String[]{"1"}, null));
        this.store.saveRoster(initialRoster);

        XmppUsersStore.Roster userRoster = this.store.readRoster(user);
        assertThat(userRoster.getRoster().size(), is(3));

        userRoster.getRoster().clear();
        this.store.saveRoster(userRoster);

        Map<BareJID, RosterElement> afterClearRoster = database.roster.of(0);
        assertThat(afterClearRoster.size(), is(0));
    }

}
