/*
 * Copyright 2013 Kerio Technologies s.r.o. All rights reserved.
 */
package com.kerio.im.service;

import com.kerio.im.common.Task;
import com.kerio.im.common.error.Code;
import com.kerio.im.core.logging.LoggingManager;
import com.kerio.im.service.task.ParseCmd;
import com.kerio.im.service.task.PrintSystemConfiguration;
import com.kerio.im.service.task.StartServer;
import java.lang.reflect.InvocationTargetException;
import java.lang.reflect.Method;
import java.util.HashMap;
import java.util.Map;
import java.util.logging.Level;
import java.util.logging.Logger;

/**
 *
 * @author tskoda
 */
public class IMService {

    private static final Logger LOG = Logger.getLogger(IMService.class.getName());

    /**
     * @param args the command line arguments
     */
    public static void main(String[] args) {
        try {
            IMService imService = new IMService(args);
            imService.start();

        } catch (Exception e) {
            //TODO just try to cast to right exception, you dont need to use generics
            Code result = Code.UNKNOWN_ERROR;
            String message = e.getMessage();
            Throwable instance = e;
            try {
                Class<?> exception;
                if (e instanceof InvocationTargetException) {
                    exception = e.getCause().getClass();
                    message = e.getCause().getMessage();
                    instance = e.getCause();
                } else {
                    exception = e.getClass();
                    message = e.getMessage();
                    instance = e;
                }

                Method method = exception.getMethod("getIntCode");

                if (method != null) {
                    result = (Code) method.invoke(instance);
                }
            } catch (Exception ie) {
                //nothing
            }

            LOG.log(Level.SEVERE, message, instance);
            System.exit(result.ordinal());
        }
    }
    private final String[] args;

    public IMService(String[] args) {
        this.args = args;
    }

    public void start() {
        Map<String, String> cmdProperties = new HashMap<String, String>();
        run(new ParseCmd(args), cmdProperties);
        
        LoggingManager.install(cmdProperties);
        LOG.log(Level.INFO, "Starting Instant Messaging service");
        
        PrintSystemConfiguration printSysConf = new PrintSystemConfiguration();
        run(printSysConf, null);
        
        StartServer startServer = new StartServer();
        run(startServer, cmdProperties);
    }

    private <T> void run(Task<T> task, T parameter) {
        task.run(parameter);
    }
}
