/*
 *  Copyright 2013 Kerio Technologies s.r.o. All rights reserved.
 */
package tigase.muc.repository;

import java.util.Map;
import tigase.muc.Room;
import tigase.muc.RoomConfig;
import tigase.muc.exceptions.MUCException;
import tigase.server.Permissions;
import tigase.util.TigaseStringprepException;
import tigase.xmpp.Authorization;
import tigase.xmpp.BareJID;
import tigase.xmpp.JID;

/**
 *
 * @author tskoda
 */
public class SecureRepository implements IMucRepository {
    
    private final IMucRepository repository;

    public SecureRepository(IMucRepository repository) {
        this.repository = repository;
    }

    @Override
    public Room createNewRoom(Permissions permission, BareJID roomJID, JID senderJid) throws RepositoryException, MUCException {
        if (Permissions.AUTH == permission) { //we do not need to check access policy, all auth users can create rooms
            return this.repository.createNewRoom(permission, roomJID, senderJid);
        } else {
            throw new MUCException(Authorization.NOT_ALLOWED);
        }
    }

    @Override
    public void destroyRoom(Room room) throws RepositoryException {
        this.repository.destroyRoom(room);
    }

    @Override
    public Map<BareJID, Room> getActiveRooms() {
        return this.repository.getActiveRooms();
    }

    @Override
    public RoomConfig getDefaultRoomConfig() throws RepositoryException {
        return this.repository.getDefaultRoomConfig();
    }

    @Override
    public BareJID[] getPublicVisibleRoomsIdList() throws RepositoryException {
        return this.repository.getPublicVisibleRoomsIdList();
    }

    @Override
    public Room getRoom(BareJID roomJID) throws RepositoryException, MUCException, TigaseStringprepException {
        return this.repository.getRoom(roomJID);
    }

    @Override
    public String getRoomName(String jid) throws RepositoryException {
        return this.repository.getRoomName(jid);
    }

    @Override
    public boolean isRoomIdExists(String newRoomName) {
        return this.repository.isRoomIdExists(newRoomName);
    }

    @Override
    public void leaveRoom(Room room) {
        this.repository.leaveRoom(room);
    }
   
}
